
import os
import sys

if sys.platform == 'win32':
    _dir = os.path.join(os.path.dirname(__file__), 'lib')
    if os.path.isdir(_dir):
        if hasattr(os, 'add_dll_directory'):
            os.add_dll_directory(_dir)
        os.environ['PATH'] = _dir + ';' + os.environ['PATH']

class Exiv2Error(Exception):
    """Python exception raised by exiv2 library errors.

    :ivar ErrorCode code: The Exiv2 error code that caused the exception.
    :ivar str message: The message associated with the exception.
    """
    def __init__(self, code, message):
        self.code= code
        self.message = message

#: python-exiv2 version as a string
__version__ = "0.17.1"
#: python-exiv2 version as a tuple of ints
__version_tuple__ = tuple((0, 17, 1))

__all__ = ["Exiv2Error"]
from exiv2.basicio import *
__all__ += exiv2._basicio.__all__
from exiv2.datasets import *
__all__ += exiv2._datasets.__all__
from exiv2.easyaccess import *
__all__ += exiv2._easyaccess.__all__
from exiv2.error import *
__all__ += exiv2._error.__all__
from exiv2.exif import *
__all__ += exiv2._exif.__all__
from exiv2.image import *
__all__ += exiv2._image.__all__
from exiv2.iptc import *
__all__ += exiv2._iptc.__all__
from exiv2.metadatum import *
__all__ += exiv2._metadatum.__all__
from exiv2.preview import *
__all__ += exiv2._preview.__all__
from exiv2.properties import *
__all__ += exiv2._properties.__all__
from exiv2.tags import *
__all__ += exiv2._tags.__all__
from exiv2.types import *
__all__ += exiv2._types.__all__
from exiv2.value import *
__all__ += exiv2._value.__all__
from exiv2.version import *
__all__ += exiv2._version.__all__
from exiv2.xmp import *
__all__ += exiv2._xmp.__all__

__all__ = [x for x in __all__ if x[0] != '_']
__all__.sort()



##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import io
import os
import sys
import tempfile
import unittest

import exiv2


class TestPropertiesModule(unittest.TestCase):
    key_name = 'Xmp.dc.description'
    namespace = 'http://purl.org/dc/elements/1.1/'
    prefix_name = 'dc'
    property_name = 'description'

    @classmethod
    def setUpClass(cls):
        exiv2.XmpParser.initialize()

    @classmethod
    def tearDownClass(cls):
        exiv2.XmpParser.terminate()

    def check_result(self, result, expected_type, expected_value):
        self.assertIsInstance(result, expected_type)
        self.assertEqual(result, expected_value)

    def test_XmpNsInfo(self):
        ns_info = exiv2.XmpProperties.nsInfo(self.prefix_name)
        self.assertIsInstance(ns_info, exiv2.XmpNsInfo)
        self.check_result(ns_info['desc'], str, 'Dublin Core schema')
        self.check_result(ns_info['ns'], str, self.namespace)
        self.check_result(ns_info['prefix'], str, self.prefix_name)
        property_info = ns_info['xmpPropertyInfo']
        self.assertIsInstance(property_info, list)
        self.assertGreater(len(property_info), 0)
        self.assertIsInstance(property_info[0], exiv2.XmpPropertyInfo)

    def test_XmpProperties(self):
        properties = exiv2.XmpProperties
        self.check_result(properties.ns(self.prefix_name), str, self.namespace)
        self.check_result(properties.nsDesc(self.prefix_name),
                          str, 'Dublin Core schema')
        self.assertIsInstance(properties.nsInfo(self.prefix_name),
                              exiv2.XmpNsInfo)
        self.check_result(properties.prefix('http://purl.org/dc/elements/1.1/'),
                          str, self.prefix_name)
        key = exiv2.XmpKey(self.key_name)
        key2 = exiv2.XmpKey('Xmp.dc.unknown')
        property_desc = properties.propertyDesc(key)
        self.assertIsInstance(property_desc, str)
        self.assertTrue(property_desc.startswith('A textual description of'))
        self.assertIsNone(properties.propertyDesc(key2))
        property_info = properties.propertyInfo(key)
        self.assertIsInstance(property_info, exiv2.XmpPropertyInfo)
        self.assertIsNone(properties.propertyInfo(key2))
        property_list = properties.propertyList(self.prefix_name)
        self.assertIsInstance(property_list, list)
        self.assertGreater(len(property_list), 0)
        self.assertIsInstance(property_list[0], exiv2.XmpPropertyInfo)
        self.check_result(properties.propertyTitle(key), str, 'Description')
        self.assertIsNone(properties.propertyTitle(key2))
        self.check_result(properties.propertyType(key),
                          exiv2.TypeId, exiv2.TypeId.langAlt)
        self.check_result(properties.propertyType(key2),
                          exiv2.TypeId, exiv2.TypeId.xmpText)
        namespaces = properties.registeredNamespaces()
        self.assertIsInstance(namespaces, dict)
        self.assertGreater(len(namespaces), 0)
        self.assertEqual(namespaces[self.prefix_name], self.namespace)
        # these don't seem to have any effect
        properties.registerNs('http://example.com/', 'exmpl')
        properties.unregisterNs('http://example.com/')
        properties.unregisterNs()

    def test_XmpPropertyInfo(self):
        key = exiv2.XmpKey(self.key_name)
        property_info = exiv2.XmpProperties.propertyInfo(key)
        self.assertIsInstance(property_info, exiv2.XmpPropertyInfo)
        desc = property_info['desc']
        self.assertIsInstance(desc, str)
        self.assertTrue(desc.startswith('A textual description of'))
        self.check_result(property_info['name'], str, 'description')
        self.check_result(property_info['title'], str, 'Description')
        self.check_result(property_info['typeId'],
                          exiv2.TypeId, exiv2.TypeId.langAlt)
        self.check_result(property_info['xmpCategory'],
                          exiv2.XmpCategory, exiv2.XmpCategory.External)
        self.check_result(property_info['xmpValueType'], str, 'Lang Alt')

    def test_XmpKey(self):
        # constructors
        key = exiv2.XmpKey(self.key_name)
        self.assertIsInstance(key, exiv2.XmpKey)
        key2 = exiv2.XmpKey(self.prefix_name, self.property_name)
        self.assertIsInstance(key2, exiv2.XmpKey)
        self.assertIsNot(key2, key)
        key2 = exiv2.XmpKey(key)
        self.assertIsInstance(key2, exiv2.XmpKey)
        self.assertIsNot(key2, key)
        # other methods
        self.assertEqual(str(key), self.key_name)
        key2 = key.clone()
        self.assertIsNot(key2, key)
        self.assertIsInstance(key2, exiv2.XmpKey)
        self.check_result(key.familyName(), str, self.key_name.split('.')[0])
        self.check_result(key.groupName(), str, self.key_name.split('.')[1])
        self.check_result(key.key(), str, self.key_name)
        self.check_result(key.ns(), str, self.namespace)
        self.check_result(key.tag(), int, 0)
        self.check_result(key.tagLabel(), str, 'Description')
        self.check_result(key.tagName(), str, self.key_name.split('.')[2])
        buf = io.StringIO()
        buf = key.write(buf)
        self.assertEqual(buf.getvalue(), self.key_name)


if __name__ == '__main__':
    unittest.main()



##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import datetime
from fractions import Fraction
import io
import os
import random
import struct
import sys
import unittest

import exiv2


class TestValueModule(unittest.TestCase):
    def check_result(self, result, expected_type, expected_value):
        self.assertIsInstance(result, expected_type)
        self.assertEqual(result, expected_value)

    def do_common_tests(self, value, type_id, string, data, sequence=None):
        if type_id != exiv2.TypeId.undefined:
            self.assertIsInstance(exiv2.Value.create(type_id), type(value))
        self.assertEqual(len(value), value.count())
        self.assertEqual(str(value), string)
        result = value.clone()
        self.assertIsInstance(result, type(value))
        self.assertEqual(str(result), str(value))
        result = bytearray(len(data))
        self.assertEqual(
            value.copy(result, exiv2.ByteOrder.littleEndian), len(result))
        self.assertEqual(result, data)
        if sequence:
            self.check_result(value.count(), int, len(sequence))
            if not isinstance(sequence, dict):
                with self.assertRaises(IndexError):
                    result = value[value.count()]
        else:
            self.check_result(value.count(), int, len(data))
        self.check_result(value.ok(), bool, True)
        with self.assertWarns(DeprecationWarning):
            result = exiv2.Value.create(int(type_id))
        result = exiv2.Value.create(type_id)
        self.assertEqual(result.read(string), 0)
        self.assertEqual(str(result), string)
        result = exiv2.Value.create(type_id)
        self.assertEqual(result.read(data, exiv2.ByteOrder.littleEndian), 0)
        self.assertEqual(str(result), string)
        self.check_result(value.size(), int, len(data))
        # Exiv2::CommentValue::typeId returns undefined
        if type_id == exiv2.TypeId.comment:
            self.check_result(
                value.typeId(), exiv2.TypeId, exiv2.TypeId.undefined)
        else:
            self.check_result(value.typeId(), exiv2.TypeId, type_id)
        buf = io.StringIO()
        buf = value.write(buf)
        self.assertEqual(buf.getvalue(), string)

    def do_conversion_tests(self, value, text, number):
        result = value.toFloat(0)
        self.assertEqual(value.ok(), True)
        self.assertIsInstance(result, float)
        self.assertAlmostEqual(result, float(number), places=5)
        self.assertEqual(value.toFloat(0), value.toFloat())
        if exiv2.testVersion(0, 28, 0):
            self.check_result(value.toUint32(0), int, int(number))
            self.assertEqual(value.ok(), True)
            self.assertEqual(value.toUint32(0), value.toUint32())
            self.check_result(value.toInt64(0), int, int(number))
            self.assertEqual(value.toInt64(0), value.toInt64())
        else:
            self.check_result(value.toLong(0), int, int(number))
            self.assertEqual(value.toLong(0), value.toLong())
        self.assertEqual(value.ok(), True)
        result = value.toRational(0)
        self.assertEqual(value.ok(), True)
        self.assertIsInstance(result, tuple)
        self.assertAlmostEqual(
            float(Fraction(*result)), float(number), places=5)
        self.assertEqual(value.toRational(0), value.toRational())
        self.check_result(value.toString(0), str, text)
        self.assertEqual(value.ok(), True)

    def do_dataarea_tests(self, value, has_dataarea=False):
        data_area = value.dataArea()
        self.assertIsInstance(data_area, exiv2.DataBuf)
        self.assertEqual(len(data_area), 0)
        if has_dataarea:
            self.assertEqual(value.setDataArea(b'fred'), 0)
            self.check_result(value.sizeDataArea(), int, 4)
        else:
            self.assertEqual(value.setDataArea(b'fred'), -1)

    def do_common_string_tests(self, value, data):
        with self.assertWarns(DeprecationWarning):
            char = value[0]
        with value.data() as view:
            self.assertIsInstance(view, memoryview)
            self.assertEqual(view, data)

    def do_common_xmp_tests(self, value):
        with self.assertWarns(DeprecationWarning):
            type_ = exiv2.XmpArrayType.xaSeq
        for type_ in (exiv2.XmpValue.XmpArrayType.xaSeq,
                      exiv2.XmpValue.XmpArrayType.xaBag,
                      exiv2.XmpValue.XmpArrayType.xaAlt,
                      exiv2.XmpValue.XmpArrayType.xaNone):
            with self.assertWarns(DeprecationWarning):
                value.setXmpArrayType(int(type_))
            value.setXmpArrayType(type_)
            self.check_result(
                value.xmpArrayType(), exiv2.XmpValue.XmpArrayType, type_)
        with self.assertWarns(DeprecationWarning):
            type_ = exiv2.XmpStruct.xsStruct
        for type_ in (exiv2.XmpValue.XmpStruct.xsStruct,
                      exiv2.XmpValue.XmpStruct.xsNone):
            value.setXmpStruct(type_)
            self.check_result(
                value.xmpStruct(), exiv2.XmpValue.XmpStruct, type_)
        value.setXmpStruct()
        self.check_result(value.xmpStruct(), exiv2.XmpValue.XmpStruct,
                          exiv2.XmpValue.XmpStruct.xsStruct)

    def test_AsciiValue(self):
        text = 'The quick brown fox jumps over the lazy dog. àéīöûç'
        data = bytes(text, 'utf-8') + b'\x00'
        # constructors
        value = exiv2.AsciiValue()
        self.assertIsInstance(value, exiv2.AsciiValue)
        self.assertEqual(len(value), 0)
        value = exiv2.AsciiValue(text)
        self.assertIsInstance(value, exiv2.AsciiValue)
        # other methods
        self.do_common_tests(value, exiv2.TypeId.asciiString, text, data)
        self.do_common_string_tests(value, data)
        self.do_conversion_tests(value, text, data[0])
        self.do_dataarea_tests(value)

    def test_CommentValue(self):
        raw_text = 'The quick brown fox jumps over the lazy dog. àéīöûç'
        data = b'UNICODE\x00' + bytes(raw_text, 'utf-16-le')
        if exiv2.testVersion(0, 27, 3):
            text = 'charset=Unicode ' + raw_text
        else:
            text = 'charset="Unicode" ' + raw_text
        # constructors
        value = exiv2.CommentValue()
        self.assertIsInstance(value, exiv2.CommentValue)
        self.assertEqual(len(value), 0)
        value = exiv2.CommentValue(text)
        self.assertIsInstance(value, exiv2.CommentValue)
        # other methods
        with self.assertWarns(DeprecationWarning):
            result = exiv2.CharsetId.ascii
        self.check_result(value.charsetId(), exiv2.CommentValue.CharsetId,
                          exiv2.CommentValue.CharsetId.unicode)
        self.check_result(value.comment(), str, raw_text)
        self.check_result(value.detectCharset(raw_text), str, 'UCS-2LE')
        self.check_result(
            value.byteOrder_, exiv2.ByteOrder, exiv2.ByteOrder.littleEndian)
        self.do_common_tests(value, exiv2.TypeId.comment, text, data)
        self.do_common_string_tests(value, data)
        self.do_conversion_tests(value, text, data[0])
        self.do_dataarea_tests(value)

    def test_StringValue(self):
        text = 'The quick brown fox jumps over the lazy dog. àéīöûç'
        data = bytes(text, 'utf-8')
        # constructors
        value = exiv2.StringValue()
        self.assertIsInstance(value, exiv2.StringValue)
        self.assertEqual(len(value), 0)
        value = exiv2.StringValue(text)
        self.assertIsInstance(value, exiv2.StringValue)
        # other methods
        self.do_common_tests(value, exiv2.TypeId.string, text, data)
        self.do_common_string_tests(value, data)
        self.do_conversion_tests(value, text, data[0])
        self.do_dataarea_tests(value)

    def test_LangAltValue(self):
        text_dict = {
            'x-default': 'The quick brown fox jumps over the lazy dog.',
            'en-GB': 'The quick brown fox jumps over the lazy dog.',
            'de-DE': 'Der schnelle Braunfuchs springt über den faulen Hund.',
            }
        text = ', '.join('lang="{}" {}'.format(*x) for x in text_dict.items())
        data = bytes(text, 'utf-8')
        # constructors
        value = exiv2.LangAltValue()
        self.assertIsInstance(value, exiv2.LangAltValue)
        self.assertEqual(len(value), 0)
        value = exiv2.LangAltValue(text_dict['x-default'])
        self.assertIsInstance(value, exiv2.LangAltValue)
        self.assertEqual(len(value), 1)
        value = exiv2.LangAltValue(text_dict)
        self.assertIsInstance(value, exiv2.LangAltValue)
        self.assertEqual(len(value), 3)
        # other methods
        self.assertEqual(dict(value), text_dict)
        self.assertEqual('de-DE' in value, True)
        self.check_result(value['en-GB'], str, text_dict['en-GB'])
        self.assertEqual('nl-NL' in value, False)
        nl_string = 'De snelle bruine vos springt over de luie hond heen.'
        value['nl-NL'] = nl_string
        self.assertEqual('nl-NL' in value, True)
        del value['nl-NL']
        self.assertEqual('nl-NL' in value, False)
        self.assertEqual(value.read(nl_string), 0)
        self.assertEqual(value['x-default'], nl_string)
        self.assertEqual(
            value.read('lang="{}" {}'.format('nl-NL', nl_string)), 0)
        self.assertEqual('nl-NL' in value, True)
        self.assertEqual(value['nl-NL'], nl_string)
        value = exiv2.LangAltValue(text_dict)
        self.check_result(value.keys(), list, list(text_dict.keys()))
        self.check_result(value.values(), list, list(text_dict.values()))
        self.check_result(value.items(), list, list(text_dict.items()))
        self.do_common_tests(
            value, exiv2.TypeId.langAlt, text, data, sequence=text_dict)
        # no conversion tests as value can't be numeric
        self.do_dataarea_tests(value)
        self.do_common_xmp_tests(value)

    def test_XmpArrayValue(self):
        text = ('The quick brown fox jumps over the lazy dog. àéīöûç',
                'Lorem ipsum dolor sit amet, consectetur adipiscing elit')
        string = ', '.join(text)
        data = bytes(string, 'utf-8')
        # constructors
        value = exiv2.XmpArrayValue()
        self.assertIsInstance(value, exiv2.XmpArrayValue)
        self.check_result(value.typeId(), exiv2.TypeId, exiv2.TypeId.xmpBag)
        self.assertEqual(len(value), 0)
        value = exiv2.XmpArrayValue(exiv2.TypeId.xmpSeq)
        self.assertIsInstance(value, exiv2.XmpArrayValue)
        self.check_result(value.typeId(), exiv2.TypeId, exiv2.TypeId.xmpSeq)
        value = exiv2.XmpArrayValue(text)
        self.assertIsInstance(value, exiv2.XmpArrayValue)
        self.check_result(value.typeId(), exiv2.TypeId, exiv2.TypeId.xmpBag)
        value = exiv2.XmpArrayValue(text, exiv2.TypeId.xmpSeq)
        self.assertIsInstance(value, exiv2.XmpArrayValue)
        self.check_result(value.typeId(), exiv2.TypeId, exiv2.TypeId.xmpSeq)
        # other methods
        self.assertEqual(len(value), 2)
        self.check_result(value[0], str, text[0])
        value.append('fred')
        self.assertIsInstance(value[2], str)
        value = exiv2.XmpArrayValue(text)
        # read() appends to value
        self.assertEqual(value.read(b'dave'), 0)
        self.assertEqual(len(value), 3)
        self.assertEqual(value[2], 'dave')
        value = exiv2.XmpArrayValue(text)
        self.assertEqual(value.read('pete'), 0)
        self.assertEqual(value[2], 'pete')
        self.assertEqual(len(value), 3)
        value = exiv2.XmpArrayValue(text)
        self.do_common_tests(
            value, exiv2.TypeId.xmpBag, string, data, sequence=text)
        value = exiv2.XmpArrayValue(('123.45', 'fred'))
        self.do_conversion_tests(value, value[0], float(value[0]))
        self.do_dataarea_tests(value)
        self.do_common_xmp_tests(value)

    def test_XmpTextValue(self):
        text = 'The quick brown fox jumps over the lazy dog. àéīöûç'
        data = bytes(text, 'utf-8')
        # constructors
        value = exiv2.XmpTextValue()
        self.assertIsInstance(value, exiv2.XmpTextValue)
        self.assertEqual(len(value), 0)
        value = exiv2.XmpTextValue(text)
        self.assertIsInstance(value, exiv2.XmpTextValue)
        # other methods
        self.do_common_tests(value, exiv2.TypeId.xmpText, text, data)
        self.do_common_string_tests(value, data)
        text = '123.45'
        value = exiv2.XmpArrayValue(text)
        self.do_conversion_tests(value, text[0], int(text[0]))
        self.do_dataarea_tests(value)
        self.do_common_xmp_tests(value)

    def test_DataValue(self):
        def check_data(value, data):
            copy = bytearray(len(data))
            self.assertEqual(value.copy(copy), len(data))
            self.assertEqual(copy, data)

        data = bytes(random.choices(range(256), k=128))
        string = ' '.join(str(x) for x in data)
        # constructors
        value = exiv2.DataValue()
        self.assertIsInstance(value, exiv2.DataValue)
        self.check_result(value.typeId(), exiv2.TypeId, exiv2.TypeId.undefined)
        self.assertEqual(len(value), 0)
        value = exiv2.DataValue(exiv2.TypeId.unsignedByte)
        self.assertIsInstance(value, exiv2.DataValue)
        self.check_result(
            value.typeId(), exiv2.TypeId, exiv2.TypeId.unsignedByte)
        self.assertEqual(len(value), 0)
        value = exiv2.DataValue(data)
        self.assertIsInstance(value, exiv2.DataValue)
        self.check_result(value.typeId(), exiv2.TypeId, exiv2.TypeId.undefined)
        check_data(value, data)
        value = exiv2.DataValue(
            data, exiv2.ByteOrder.invalidByteOrder, exiv2.TypeId.unsignedByte)
        self.assertIsInstance(value, exiv2.DataValue)
        self.check_result(
            value.typeId(), exiv2.TypeId, exiv2.TypeId.unsignedByte)
        check_data(value, data)
        # other methods
        self.do_common_tests(value, exiv2.TypeId.unsignedByte, string, data)
        self.do_conversion_tests(value, str(data[0]), data[0])
        self.do_dataarea_tests(value)

    def test_Date(self):
        today = datetime.date.today()
        value = exiv2.Date()
        value.year = today.year
        value.month = today.month
        value.day = today.day
        self.assertIsInstance(value, exiv2.Date)
        self.assertIsInstance(value.year, int)
        self.assertEqual(value.year, today.year)
        self.assertIsInstance(value.month, int)
        self.assertEqual(value.month, today.month)
        self.assertIsInstance(value.day, int)
        self.assertEqual(value.day, today.day)
        self.assertEqual(dict(value), {
            'year': today.year, 'month': today.month, 'day': today.day})

    def test_DateValue(self):
        py_date = datetime.date.today()
        data = bytes(py_date.strftime('%Y%m%d'), 'ascii')
        exiv_date = exiv2.Date()
        exiv_date.year = py_date.year
        exiv_date.month = py_date.month
        exiv_date.day = py_date.day
        date_dict = {
            'year': py_date.year, 'month': py_date.month, 'day': py_date.day}
        # constructors
        value = exiv2.DateValue()
        self.assertIsInstance(value, exiv2.DateValue)
        value = exiv2.DateValue(exiv_date)
        self.assertIsInstance(value, exiv2.DateValue)
        self.assertEqual(str(value), py_date.isoformat())
        value = exiv2.DateValue(py_date.year, py_date.month, py_date.day)
        self.assertIsInstance(value, exiv2.DateValue)
        self.assertEqual(str(value), py_date.isoformat())
        # other methods
        with self.assertWarns(DeprecationWarning):
            result = value[0]
        self.check_result(dict(value.getDate()), dict, date_dict)
        value.setDate(exiv_date)
        self.assertEqual(str(value), py_date.isoformat())
        value.setDate(py_date.year, py_date.month, py_date.day)
        self.assertEqual(str(value), py_date.isoformat())
        seconds = int(datetime.datetime.combine(
            py_date, datetime.time(), datetime.timezone.utc).timestamp())
        self.do_common_tests(value, exiv2.TypeId.date, py_date.isoformat(), data)
        self.do_conversion_tests(value, py_date.isoformat(), seconds)
        self.do_dataarea_tests(value)

    def test_Time(self):
        now = datetime.datetime.now().time()
        value = exiv2.Time()
        value.hour = now.hour
        value.minute = now.minute
        value.second = now.second
        value.tzHour = 1
        value.tzMinute = 30
        self.assertIsInstance(value, exiv2.Time)
        self.assertIsInstance(value.hour, int)
        self.assertEqual(value.hour, now.hour)
        self.assertIsInstance(value.minute, int)
        self.assertEqual(value.minute, now.minute)
        self.assertIsInstance(value.second, int)
        self.assertEqual(value.second, now.second)
        self.assertIsInstance(value.tzHour, int)
        self.assertEqual(value.tzHour, 1)
        self.assertIsInstance(value.tzMinute, int)
        self.assertEqual(value.tzMinute, 30)
        self.assertEqual(dict(value), {
            'hour': now.hour, 'minute': now.minute, 'second': now.second,
            'tzHour': 1, 'tzMinute': 30})

    def test_TimeValue(self):
        py_tz = datetime.timezone(datetime.timedelta(hours=1, minutes=30))
        py_time = datetime.datetime.now().time()
        py_time = py_time.replace(microsecond=0, tzinfo=py_tz)
        exiv_time = exiv2.Time()
        exiv_time.hour = py_time.hour
        exiv_time.minute = py_time.minute
        exiv_time.second = py_time.second
        exiv_time.tzHour = 1
        exiv_time.tzMinute = 30
        time_dict = {'hour': py_time.hour, 'minute': py_time.minute,
                     'second': py_time.second, 'tzHour': 1, 'tzMinute': 30}
        data = bytes(py_time.strftime('%H%M%S%z'), 'ascii')
        # constructors
        value = exiv2.TimeValue()
        self.assertIsInstance(value, exiv2.TimeValue)
        value = exiv2.TimeValue(exiv_time)
        self.assertIsInstance(value, exiv2.TimeValue)
        self.assertEqual(str(value), py_time.isoformat())
        value = exiv2.TimeValue(py_time.hour, py_time.minute, py_time.second)
        self.assertIsInstance(value, exiv2.TimeValue)
        value = exiv2.TimeValue(
            py_time.hour, py_time.minute, py_time.second, 1)
        self.assertIsInstance(value, exiv2.TimeValue)
        value = exiv2.TimeValue(
            py_time.hour, py_time.minute, py_time.second, 1, 30)
        self.assertIsInstance(value, exiv2.TimeValue)
        self.assertEqual(str(value), py_time.isoformat())
        # other methods
        with self.assertWarns(DeprecationWarning):
            result = value[0]
        self.check_result(dict(value.getTime()), dict, time_dict)
        value.setTime(exiv_time)
        self.assertEqual(str(value), py_time.isoformat())
        value.setTime(py_time.hour, py_time.minute)
        value.setTime(py_time.hour, py_time.minute, py_time.second)
        value.setTime(py_time.hour, py_time.minute, py_time.second, 1)
        value.setTime(py_time.hour, py_time.minute, py_time.second, 1, 30)
        self.assertEqual(str(value), py_time.isoformat())
        seconds = (py_time.hour * 3600) + (py_time.minute * 60) + py_time.second
        seconds -= 3600 + (30 * 60)
        value = exiv2.TimeValue()
        value.read(py_time.isoformat())
        self.do_common_tests(value, exiv2.TypeId.time, py_time.isoformat(), data)
        self.do_conversion_tests(value, py_time.isoformat(), seconds)
        self.do_dataarea_tests(value)

    def test_UShortValue(self):
        sequence = (4, 6, 9, 5)
        text = ' '.join(str(x) for x in sequence)
        data = struct.pack('<4H', *sequence)
        # constructors
        value = exiv2.UShortValue()
        self.assertIsInstance(value, exiv2.UShortValue)
        self.assertEqual(len(value), 0)
        value = exiv2.UShortValue(sequence)
        self.assertIsInstance(value, exiv2.UShortValue)
        self.assertEqual(tuple(value), sequence)
        # data access
        self.check_result(value[0], int, sequence[0])
        self.check_result(value[2], int, sequence[2])
        self.check_result(value[-1], int, sequence[-1])
        value[2] = 56
        self.assertEqual(value[2], 56)
        del value[2]
        self.assertEqual(len(value), len(sequence) - 1)
        value.append(3)
        self.assertEqual(value[3], 3)
        # other methods
        value = exiv2.UShortValue(sequence)
        self.do_common_tests(
            value, exiv2.TypeId.unsignedShort, text, data, sequence=sequence)
        self.do_conversion_tests(value, str(sequence[0]), sequence[0])
        self.do_dataarea_tests(value, has_dataarea=True)
        # data range
        value = exiv2.UShortValue((1<<16) - 1)
        with self.assertRaises(TypeError):
            value = exiv2.UShortValue(1<<16)
        value = exiv2.UShortValue(0)
        with self.assertRaises(TypeError):
            value = exiv2.UShortValue(-1)

    def test_URationalValue(self):
        sequence = ((4, 3), (7, 13), (23, 3))
        text = ' '.join('{}/{}'.format(x, y) for x, y in sequence)
        data = struct.pack('<6I', *[x for y in sequence for x in y])
        # constructors
        value = exiv2.URationalValue()
        self.assertIsInstance(value, exiv2.URationalValue)
        self.assertEqual(len(value), 0)
        value = exiv2.URationalValue(sequence)
        self.assertIsInstance(value, exiv2.URationalValue)
        self.assertEqual(tuple(value), sequence)
        # data access
        self.check_result(value[0], tuple, sequence[0])
        self.check_result(value[1], tuple, sequence[1])
        self.check_result(value[-1], tuple, sequence[-1])
        value[2] = (56, 13)
        self.assertEqual(value[2], (56, 13))
        del value[2]
        self.assertEqual(len(value), len(sequence) - 1)
        value.append((3, 1))
        self.assertEqual(value[2], (3, 1))
        # other methods
        value = exiv2.URationalValue(sequence)
        self.do_common_tests(
            value, exiv2.TypeId.unsignedRational, text, data, sequence=sequence)
        self.do_conversion_tests(
            value, '{}/{}'.format(*sequence[0]), Fraction(*sequence[0]))
        self.do_dataarea_tests(value, has_dataarea=True)
        # data range
        value = exiv2.URationalValue([((1<<32) - 1, 1)])
        with self.assertRaises(TypeError):
            value = exiv2.URationalValue([(1<<32, 1)])
        value = exiv2.URationalValue([(0, 1)])
        with self.assertRaises(TypeError):
            value = exiv2.URationalValue([(-1, 1)])


if __name__ == '__main__':
    unittest.main()



##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import os
import sys
import tempfile
import unittest

import exiv2


class TestImageModule(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        test_dir = os.path.dirname(__file__)
        cls.image_path = os.path.join(test_dir, 'image_02.jpg')
        cls.bmff_path = os.path.join(test_dir, 'image_02.heic')
        # read image file data into memory
        with open(cls.image_path, 'rb') as f:
            cls.image_data = f.read()

    def check_result(self, result, expected_type, expected_value):
        self.assertIsInstance(result, expected_type)
        self.assertEqual(result, expected_value)

    def test_BMFF(self):
        with self.assertWarns(DeprecationWarning):
            enabled = exiv2.enableBMFF(True)
        self.assertEqual(enabled, exiv2.versionInfo()['EXV_ENABLE_BMFF'])
        with open(self.bmff_path, 'rb') as f:
            image_data = f.read()
        if not exiv2.versionInfo()['EXV_ENABLE_BMFF']:
            if (exiv2.testVersion(0, 28, 0)
                    or not exiv2.versionInfo()['EXV_ENABLE_VIDEO']):
                with self.assertRaises(exiv2.Exiv2Error) as cm:
                    image = exiv2.ImageFactory.open(image_data)
                self.assertEqual(cm.exception.code,
                                 exiv2.ErrorCode.kerMemoryContainsUnknownImageType)
            self.skipTest('EXV_ENABLE_BMFF is off')
        image = exiv2.ImageFactory.open(image_data)
        image.readMetadata()
        self.assertEqual(len(image.exifData()), 29)
        self.assertEqual(len(image.iptcData()), 0)
        self.assertEqual(len(image.xmpData()), 26)
        self.assertEqual(len(image.iccProfile()), 672)

    def test_Image(self):
        # open image in memory so we don't corrupt the file
        image = exiv2.ImageFactory.open(self.image_data)
        self.assertEqual(len(image.io()), 15125)
        # test clearMetadata
        image.readMetadata()
        self.check_result(image.comment(), str, 'Created with GIMP')
        self.assertEqual(len(image.exifData()), 29)
        self.assertEqual(len(image.iptcData()), 19)
        self.assertEqual(len(image.xmpData()), 26)
        self.assertEqual(len(image.iccProfile()), 672)
        image.clearMetadata()
        self.check_result(image.comment(), str, '')
        self.assertEqual(len(image.exifData()), 0)
        self.assertEqual(len(image.iptcData()), 0)
        self.assertEqual(len(image.xmpData()), 0)
        self.assertEqual(len(image.iccProfile()), 0)
        image.writeMetadata()
        self.assertEqual(len(image.io()), 6371)
        # test setting individual parts
        image2 = exiv2.ImageFactory.open(self.image_data)
        image2.readMetadata()
        image.setComment(image2.comment())
        self.check_result(image.comment(), str, 'Created with GIMP')
        image.setExifData(image2.exifData())
        self.assertEqual(len(image.exifData()), 29)
        image.setIptcData(image2.iptcData())
        self.assertEqual(len(image.iptcData()), 19)
        image.setXmpPacket(image2.xmpPacket())
        self.assertEqual(len(image.xmpPacket()), 4234)
        image.setXmpData(image2.xmpData())
        self.assertEqual(len(image.xmpData()), 26)
        image.setIccProfile(image2.iccProfile())
        self.assertEqual(len(image.iccProfile()), 672)
        image.writeMetadata()
        self.assertEqual(len(image.io()), 15125)
        del image2
        # test clearing individual parts
        image = exiv2.ImageFactory.open(self.image_data)
        image.readMetadata()
        self.check_result(image.comment(), str, 'Created with GIMP')
        image.clearComment()
        self.check_result(image.comment(), str, '')
        self.assertEqual(len(image.exifData()), 29)
        image.clearExifData()
        self.assertEqual(len(image.exifData()), 0)
        self.assertEqual(len(image.iptcData()), 19)
        image.clearIptcData()
        self.assertEqual(len(image.iptcData()), 0)
        self.assertEqual(len(image.xmpPacket()), 4234)
        image.clearXmpPacket()
        self.check_result(image.xmpPacket(), str, '')
        self.assertEqual(len(image.xmpData()), 26)
        image.clearXmpData()
        self.assertEqual(len(image.xmpData()), 0)
        self.assertEqual(len(image.iccProfile()), 672)
        image.clearIccProfile()
        self.assertEqual(len(image.iccProfile()), 0)
        # test other methods
        image.readMetadata()
        self.check_result(image.byteOrder(),
                          exiv2.ByteOrder, exiv2.ByteOrder.littleEndian)
        self.check_result(image.checkMode(exiv2.MetadataId.Exif),
                          exiv2.AccessMode, exiv2.AccessMode.ReadWrite)
        self.check_result(image.good(), bool, True)
        self.check_result(image.iccProfileDefined(), bool, True)
        self.check_result(image.imageType(),
                          exiv2.ImageType, exiv2.ImageType.jpeg)
        self.assertIsInstance(image.io(), exiv2.BasicIo)
        self.check_result(image.mimeType(), str, 'image/jpeg')
        self.check_result(image.pixelHeight(), int, 200)
        self.check_result(image.pixelWidth(), int, 200)
        image.setByteOrder(exiv2.ByteOrder.littleEndian)
        image.setComment('fred')
        self.check_result(image.comment(), str, 'fred')

    def test_ImageFactory(self):
        factory = exiv2.ImageFactory
        with self.assertWarns(DeprecationWarning):
            factory.checkMode(int(exiv2.ImageType.jpeg), exiv2.MetadataId.Exif)
        self.check_result(
            factory.checkMode(exiv2.ImageType.jpeg, exiv2.MetadataId.Exif),
            exiv2.AccessMode, exiv2.AccessMode.ReadWrite)
        io = factory.createIo(self.image_data)
        with self.assertWarns(DeprecationWarning):
            factory.checkType(int(exiv2.ImageType.jpeg), io, False)
        self.check_result(
            factory.checkType(exiv2.ImageType.jpeg, io, False), bool, True)
        with self.assertWarns(DeprecationWarning):
            factory.create(int(exiv2.ImageType.jpeg))
        self.assertIsInstance(
            factory.create(exiv2.ImageType.jpeg), exiv2.Image)
        self.assertIsInstance(factory.createIo(self.image_data), exiv2.BasicIo)
        self.check_result(factory.getType(self.image_data),
                          exiv2.ImageType, exiv2.ImageType.jpeg)
        self.check_result(factory.getType(io),
                          exiv2.ImageType, exiv2.ImageType.jpeg)
        self.assertIsInstance(factory.open(self.image_data), exiv2.Image)
        if not exiv2.versionInfo()['EXV_ENABLE_FILESYSTEM']:
            self.skipTest('EXV_ENABLE_FILESYSTEM is off')
        with tempfile.TemporaryDirectory() as tmp_dir:
            temp_file = os.path.join(tmp_dir, 'image.jpg')
            self.assertIsInstance(
                factory.create(exiv2.ImageType.jpeg, temp_file), exiv2.Image)
        self.assertIsInstance(factory.createIo(self.image_path), exiv2.BasicIo)
        self.check_result(factory.getType(self.image_path),
                          exiv2.ImageType, exiv2.ImageType.jpeg)
        self.assertIsInstance(factory.open(self.image_path), exiv2.Image)

    def test_ref_counts(self):
        # opening from data keeps reference to buffer
        self.assertEqual(sys.getrefcount(self.image_data), 2)
        image = exiv2.ImageFactory.open(self.image_data)
        self.assertEqual(sys.getrefcount(self.image_data), 3)
        # writeMetadata releases buffer
        image.writeMetadata()
        self.assertEqual(sys.getrefcount(self.image_data), 2)
        # getting metadata and buffers keeps reference to image
        image = exiv2.ImageFactory.open(self.image_data)
        image.readMetadata()
        self.assertEqual(sys.getrefcount(image), 2)
        data = [image.exifData()]
        self.assertEqual(sys.getrefcount(image), 3)
        data.append(image.iptcData())
        self.assertEqual(sys.getrefcount(image), 4)
        data.append(image.xmpData())
        self.assertEqual(sys.getrefcount(image), 5)
        data.append(image.iccProfile())
        self.assertEqual(sys.getrefcount(image), 6)
        data.append(image.io())
        self.assertEqual(sys.getrefcount(image), 7)
        del data
        self.assertEqual(sys.getrefcount(image), 2)


if __name__ == '__main__':
    unittest.main()



##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import locale
import os
import shutil
import sys
import tempfile
import unittest

import exiv2


class TestBasicIoModule(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        test_dir = os.path.dirname(__file__)
        cls.image_path = os.path.join(test_dir, 'image_02.jpg')
        cls.data = b'The quick brown fox jumps over the lazy dog'

    @unittest.skipUnless(exiv2.versionInfo()['EXV_USE_CURL'],
                         'CurlIo not included')
    def test_CurlIo(self):
        https_image = ('https://raw.githubusercontent.com/jim-easterbrook'
                       '/python-exiv2/main/tests/image_02.jpg')
        io = exiv2.ImageFactory.createIo(https_image)
        self.assertIsInstance(io, exiv2.BasicIo)
        self.assertEqual(io.ioType(), 'CurlIo')
        self.assertEqual(io.error(), False)
        self.assertEqual(io.path(), https_image)
        self.assertEqual(io.size(), 0)
        # open and close
        self.assertEqual(io.isopen(), False)
        self.assertEqual(io.open(), 0)
        self.assertEqual(io.size(), 15125)
        self.assertEqual(io.isopen(), True)
        self.assertEqual(io.close(), 0)
        self.assertEqual(io.isopen(), True)

    @unittest.skipUnless(exiv2.versionInfo()['EXV_ENABLE_FILESYSTEM'],
                         'EXV_ENABLE_FILESYSTEM is off')
    def test_FileIo(self):
        # most functions are tested in test_MemIo
        io = exiv2.ImageFactory.createIo(self.image_path)
        self.assertIsInstance(io, exiv2.BasicIo)
        self.assertEqual(io.ioType(), 'FileIo')
        self.assertEqual(io.error(), False)
        self.assertEqual(io.path(), self.image_path)
        self.assertEqual(io.size(), 15125)
        # open and close
        self.assertEqual(io.isopen(), False)
        self.assertEqual(io.open(), 0)
        self.assertEqual(io.isopen(), True)
        self.assertEqual(io.close(), 0)
        self.assertEqual(io.isopen(), False)

    def test_MemIo(self):
        # empty buffer
        io = exiv2.ImageFactory.createIo(b'')
        self.assertIsInstance(io, exiv2.BasicIo)
        self.assertEqual(io.ioType(), 'MemIo')
        self.assertEqual(io.size(), 0)
        # mmap data access
        with io.mmap(False) as view:
            self.assertIsInstance(view, memoryview)
            self.assertEqual(view, b'')
            self.assertEqual(view.readonly, True)
            with self.assertRaises(TypeError):
                view[0] = 0
        self.assertEqual(io.munmap(), 0)
        with io.mmap(True) as view:
            self.assertIsInstance(view, memoryview)
            self.assertEqual(view, b'')
            self.assertEqual(view.readonly, False)
            with self.assertRaises(IndexError):
                view[0] = 0
        self.assertEqual(io.munmap(), 0)
        # Python buffer interface
        with memoryview(io) as view:
            self.assertIsInstance(view, memoryview)
            self.assertEqual(view, b'')
            self.assertEqual(view.readonly, False)
            with self.assertRaises(IndexError):
                view[0] = 0
        # non-empty buffer
        io = exiv2.ImageFactory.createIo(self.data)
        self.assertIsInstance(io, exiv2.BasicIo)
        self.assertEqual(io.ioType(), 'MemIo')
        self.assertEqual(io.error(), False)
        self.assertEqual(io.path(), 'MemIo')
        self.assertEqual(io.size(), len(self.data))
        self.assertEqual(len(io), len(self.data))
        # open and close, MemIo is always open
        self.assertEqual(io.isopen(), True)
        self.assertEqual(io.close(), 0)
        self.assertEqual(io.isopen(), True)
        self.assertEqual(io.open(), 0)
        self.assertEqual(io.isopen(), True)
        # mmap data access
        with io.mmap() as view:
            self.assertIsInstance(view, memoryview)
            self.assertEqual(view, self.data)
        self.assertEqual(io.munmap(), 0)
        # Python buffer interface
        with memoryview(io) as view:
            self.assertEqual(view, self.data)
            self.assertEqual(view.readonly, False)
        # seek & tell
        with self.assertWarns(DeprecationWarning):
            self.assertEqual(io.seek(0, exiv2.Position.beg), 0)
        self.assertEqual(io.tell(), 0)
        if exiv2.testVersion(0, 28, 0):
            self.assertEqual(
                io.seek(len(self.data) + 10, exiv2.BasicIo.Position.beg),
                exiv2.ErrorCode.kerGeneralError)
            with self.assertRaises(exiv2.Exiv2Error) as cm:
                io.seekOrThrow(len(self.data) + 10, exiv2.BasicIo.Position.beg)
            self.assertEqual(cm.exception.code,
                             exiv2.ErrorCode.kerCorruptedMetadata)
        else:
            self.assertEqual(
                io.seek(len(self.data) + 10, exiv2.BasicIo.Position.beg),
                exiv2.ErrorCode.kerErrorMessage)
        self.assertEqual(io.seek(0, exiv2.BasicIo.Position.end), 0)
        self.assertEqual(io.tell(), len(self.data))
        # reading data
        self.assertEqual(io.seek(0, exiv2.BasicIo.Position.beg), 0)
        self.assertEqual(io.getb(), self.data[0])
        self.assertEqual(io.tell(), 1)
        self.assertEqual(memoryview(io.read(10000)), self.data[1:])
        self.assertEqual(io.tell(), len(self.data))
        self.assertEqual(io.eof(), True)
        self.assertEqual(io.seek(0, exiv2.BasicIo.Position.beg), 0)
        buf = bytearray(len(self.data))
        self.assertEqual(io.read(buf, len(self.data)), len(self.data))
        self.assertEqual(buf, self.data)
        if exiv2.testVersion(0, 28, 0):
            with self.assertRaises(exiv2.Exiv2Error) as cm:
                io.readOrThrow(buf, len(self.data))
            self.assertEqual(cm.exception.code,
                             exiv2.ErrorCode.kerCorruptedMetadata)
        self.assertEqual(io.tell(), len(self.data))
        self.assertEqual(io.getb(), -1)
        # writing data
        self.assertEqual(io.putb(ord('+')), ord('+'))
        self.assertEqual(io.eof(), True)
        self.assertEqual(len(io), len(self.data) + 1)
        self.assertEqual(io.write(exiv2.ImageFactory.createIo(b'fred')), 4)
        self.assertEqual(len(io), len(self.data) + 5)
        self.assertEqual(io.write(b'+jim'), 4)
        self.assertEqual(len(io), len(self.data) + 9)
        self.assertEqual(memoryview(io), self.data + b'+fred+jim')

    def test_ref_counts(self):
        # MemIo keeps a reference to the data buffer
        self.assertEqual(sys.getrefcount(self.data), 3)
        io = exiv2.ImageFactory.createIo(self.data)
        self.assertEqual(sys.getrefcount(self.data), 4)
        del io
        self.assertEqual(sys.getrefcount(self.data), 3)

    @unittest.skipUnless(exiv2.versionInfo()['EXV_ENABLE_FILESYSTEM'],
                         'EXV_ENABLE_FILESYSTEM is off')
    def test_unicode_paths(self):
        cp = locale.getpreferredencoding()
        with tempfile.TemporaryDirectory() as tmp_dir:
            for file_name, codes in (
                    ('Lâtín.jpg', ('UTF-8', 'cp65001', 'cp1252', 'cp850')),
                    ('Русский.jpg', ('UTF-8', 'cp65001', 'cp20866', 'cp20880',
                                     'cp866', 'cp855')),
                    ('中国人.jpg', ('UTF-8', 'cp65001', 'cp54936', 'cp936'))):
                with self.subTest(file_name=file_name, codes=codes):
                    tmp_path = os.path.normcase(os.path.join(tmp_dir, file_name))
                    shutil.copyfile(self.image_path, tmp_path)
                    io = exiv2.ImageFactory.createIo(tmp_path)
                    if cp in codes:
                        self.assertEqual(io.path(), tmp_path)
                    else:
                        self.skipTest("code page '{}' not suitable for file "
                                      "name '{}'".format(cp, file_name))


if __name__ == '__main__':
    unittest.main()



##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import io
import os
import sys
import tempfile
import unittest

import exiv2


class TestExifModule(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        test_dir = os.path.dirname(__file__)
        # open image in memory so we don't corrupt the file
        with open(os.path.join(test_dir, 'image_02.jpg'), 'rb') as f:
            cls.image = exiv2.ImageFactory.open(f.read())
        # clear locale
        name = 'en_US.UTF-8'
        os.environ['LC_ALL'] = name
        os.environ['LANG'] = name
        os.environ['LANGUAGE'] = name

    def test_ExifData(self):
        # empty container
        data = exiv2.ExifData()
        self.assertEqual(len(data), 0)
        # actual data
        self.image.readMetadata()
        data = self.image.exifData()
        self.assertEqual(len(data), 29)
        # add data
        data.add(exiv2.Exifdatum(
            exiv2.ExifKey('Exif.Image.Make'), exiv2.AsciiValue('Acme')))
        self.assertEqual('Exif.Image.Make' in data, True)
        self.assertIsInstance(data['Exif.Image.Make'], exiv2.Exifdatum)
        data.add(exiv2.ExifKey('Exif.Image.Model'), exiv2.AsciiValue('Camera'))
        self.assertEqual('Exif.Image.Model' in data, True)
        self.assertIsInstance(data['Exif.Image.Model'], exiv2.Exifdatum)
        # iterators
        b = iter(data)
        self.assertIsInstance(b, exiv2.ExifData_iterator)
        self.assertEqual(b.key(), 'Exif.Image.ProcessingSoftware')
        b = data.begin()
        self.assertIsInstance(b, exiv2.ExifData_iterator)
        self.assertEqual(b.key(), 'Exif.Image.ProcessingSoftware')
        next(b)
        self.assertEqual(b.key(), 'Exif.Image.ImageDescription')
        e = data.end()
        self.assertIsInstance(e, exiv2.ExifData_iterator_base)
        k = data.findKey(exiv2.ExifKey('Exif.Photo.FocalLength'))
        self.assertIsInstance(k, exiv2.ExifData_iterator)
        self.assertEqual(k.key(), 'Exif.Photo.FocalLength')
        k = data.erase(k)
        self.assertIsInstance(k, exiv2.ExifData_iterator)
        self.assertEqual(k.key(), 'Exif.Photo.SubSecTime')
        k = data.erase(k, e)
        self.assertIsInstance(k, exiv2.ExifData_iterator_base)
        b = data.begin()
        e = data.end()
        self.assertIsInstance(str(b), str)
        self.assertIsInstance(str(e), str)
        count = 0
        while b != e:
            next(b)
            count += 1
        self.assertEqual(count, 11)
        count = 0
        for d in data:
            count += 1
        self.assertEqual(count, 11)
        count = len(list(data))
        self.assertEqual(count, 11)
        # access by key
        self.assertEqual('Exif.Image.Artist' in data, True)
        self.assertIsInstance(data['Exif.Image.Artist'], exiv2.Exifdatum)
        del data['Exif.Image.Artist']
        self.assertEqual('Exif.Image.Artist' in data, False)
        data['Exif.Image.Artist'] = 'Fred'
        self.assertEqual('Exif.Image.Artist' in data, True)
        self.assertIsInstance(data['Exif.Image.Artist'], exiv2.Exifdatum)
        with self.assertRaises(TypeError):
            data['Exif.Image.Orientation'] = 2.5
        data['Exif.Image.Orientation'] = 4
        data['Exif.Image.Orientation'] = exiv2.UShortValue(4)
        self.assertEqual('Exif.Image.Orientation' in data, True)
        self.assertIsInstance(data['Exif.Image.Orientation'], exiv2.Exifdatum)
        # sorting
        data.sortByKey()
        self.assertEqual(data.begin().key(), 'Exif.Image.Artist')
        data.sortByTag()
        self.assertEqual(data.begin().key(), 'Exif.Image.ProcessingSoftware')
        # other methods
        self.assertEqual(data.count(), 11)
        self.assertEqual(data.empty(), False)
        data.clear()
        self.assertEqual(len(data), 0)
        self.assertEqual(data.empty(), True)

    def _test_datum(self, datum):
        self.assertIsInstance(str(datum), str)
        buf = bytearray(datum.count())
        self.assertEqual(
            datum.copy(buf, exiv2.ByteOrder.littleEndian), len(buf))
        self.assertEqual(buf, bytes(datum.toString(), 'ascii'))
        self.assertEqual(datum.count(), 28)
        data_area = datum.dataArea()
        self.assertIsInstance(data_area, exiv2.DataBuf)
        self.assertEqual(len(data_area), 0)
        self.assertEqual(datum.familyName(), 'Exif')
        self.assertIsInstance(datum.getValue(), exiv2.AsciiValue)
        with self.assertWarns(DeprecationWarning):
            self.assertIsInstance(
                datum.getValue(exiv2.TypeId.asciiString), exiv2.AsciiValue)
        self.assertEqual(datum.groupName(), 'Image')
        self.assertEqual(datum.idx(), 2)
        self.assertEqual(datum.ifdName(), 'IFD0')
        self.assertEqual(datum.key(), 'Exif.Image.ImageDescription')
        self.assertEqual(datum.print(), 'Good view of the lighthouse.')
        with self.assertWarns(DeprecationWarning):
            self.assertEqual(datum._print(), datum.print())
        self.assertEqual(datum.setDataArea(b'fred'), -1)
        self.assertEqual(datum.size(), 28)
        self.assertEqual(datum.sizeDataArea(), 0)
        self.assertEqual(datum.tag(), 270)
        self.assertEqual(datum.tagLabel(), 'Image Description')
        self.assertEqual(datum.tagName(), 'ImageDescription')
        self.assertEqual(datum.toFloat(0), 71.0)
        if exiv2.testVersion(0, 28, 0):
            self.assertEqual(datum.toInt64(0), 71)
        else:
            self.assertEqual(datum.toLong(0), 71)
        self.assertEqual(datum.toRational(0), (71, 1))
        self.assertEqual(datum.toString(), 'Good view of the lighthouse.')
        self.assertEqual(datum.toString(0), 'Good view of the lighthouse.')
        self.assertEqual(datum.typeId(), exiv2.TypeId.asciiString)
        self.assertEqual(datum.typeName(), 'Ascii')
        self.assertEqual(datum.typeSize(), 1)
        self.assertIsInstance(datum.value(), exiv2.AsciiValue)
        with self.assertWarns(DeprecationWarning):
            self.assertIsInstance(
                datum.value(exiv2.TypeId.asciiString), exiv2.AsciiValue)
        buf = io.StringIO()
        buf = datum.write(buf)
        self.assertEqual(buf.getvalue(), 'Good view of the lighthouse.')
        datum.setValue('fred')
        datum.setValue(exiv2.AsciiValue('Acme'))
        with self.assertRaises(TypeError):
            datum.setValue(123)

    def test_ExifData_iterator(self):
        self.image.readMetadata()
        data = self.image.exifData()
        datum_iter = data.findKey(exiv2.ExifKey('Exif.Image.ImageDescription'))
        self.assertIsInstance(datum_iter, exiv2.ExifData_iterator)
        self.assertIsInstance(iter(datum_iter), exiv2.ExifData_iterator)
        self._test_datum(datum_iter)

    def test_ExifDatum(self):
        datum = exiv2.Exifdatum(
            exiv2.ExifKey('Exif.Image.Make'), exiv2.AsciiValue('Acme'))
        self.assertIsInstance(datum, exiv2.Exifdatum)
        datum2 = exiv2.Exifdatum(datum)
        self.assertIsInstance(datum2, exiv2.Exifdatum)
        del datum2
        self.image.readMetadata()
        data = self.image.exifData()
        datum = data['Exif.Image.ImageDescription']
        self.assertIsInstance(datum, exiv2.Exifdatum)
        self._test_datum(datum)

    def test_ExifThumb(self):
        self.image.readMetadata()
        thumb = exiv2.ExifThumb(self.image.exifData())
        self.assertIsInstance(thumb, exiv2.ExifThumb)
        data = thumb.copy()
        self.assertIsInstance(data, exiv2.DataBuf)
        self.assertEqual(len(data), 2532)
        self.assertEqual(thumb.extension(), '.jpg')
        self.assertEqual(thumb.mimeType(), 'image/jpeg')
        thumb.erase()
        self.assertEqual(len(thumb.copy()), 0)
        exif_data = exiv2.ExifData()
        thumb = exiv2.ExifThumb(exif_data)
        thumb.setJpegThumbnail(data)
        self.assertEqual(len(thumb.copy()), 2532)
        thumb.erase()
        self.assertEqual(len(thumb.copy()), 0)
        thumb.setJpegThumbnail(
            data, exiv2.URational((160, 1)), exiv2.URational((120, 1)), 1)
        self.assertEqual(len(thumb.copy()), 2532)
        if not exiv2.versionInfo()['EXV_ENABLE_FILESYSTEM']:
            self.skipTest('EXV_ENABLE_FILESYSTEM is off')
        with tempfile.TemporaryDirectory() as tmp_dir:
            temp_file = os.path.join(tmp_dir, 'thumb')
            self.assertEqual(thumb.writeFile(temp_file), 2532)
            temp_file += thumb.extension()
            thumb.erase()
            self.assertEqual(len(thumb.copy()), 0)
            thumb.setJpegThumbnail(temp_file)
            self.assertEqual(len(thumb.copy()), 2532)
            thumb.erase()
            self.assertEqual(len(thumb.copy()), 0)
            thumb.setJpegThumbnail(temp_file, exiv2.URational((160, 1)),
                                   exiv2.URational((120, 1)), 1)
            self.assertEqual(len(thumb.copy()), 2532)

    def test_ref_counts(self):
        self.image.readMetadata()
        # exifData keeps a reference to image
        self.assertEqual(sys.getrefcount(self.image), 2)
        data = self.image.exifData()
        self.assertEqual(sys.getrefcount(self.image), 3)
        # thumbnail keeps a reference to exifData
        self.assertEqual(sys.getrefcount(data), 2)
        thumb = exiv2.ExifThumb(data)
        self.assertEqual(sys.getrefcount(data), 3)
        del thumb
        self.assertEqual(sys.getrefcount(data), 2)
        # iterator keeps a reference to data
        self.assertEqual(sys.getrefcount(data), 2)
        b = data.begin()
        self.assertEqual(sys.getrefcount(data), 3)
        e = data.end()
        self.assertEqual(sys.getrefcount(data), 4)
        i = iter(data)
        self.assertEqual(sys.getrefcount(data), 5)
        k = data.findKey(exiv2.ExifKey('Exif.Photo.FocalLength'))
        self.assertEqual(sys.getrefcount(data), 6)
        k2 = data.erase(k)
        self.assertEqual(sys.getrefcount(data), 7)
        del b, e, i, k, k2
        self.assertEqual(sys.getrefcount(data), 2)
        # iterator of an iterator keeps a reference to iterator
        b = data.begin()
        self.assertEqual(sys.getrefcount(b), 2)
        i = iter(b)
        self.assertEqual(sys.getrefcount(b), 3)
        del i
        self.assertEqual(sys.getrefcount(b), 2)
        # iterator value keeps a reference to iterator
        v = b.value()
        self.assertEqual(sys.getrefcount(b), 3)
        del v
        self.assertEqual(sys.getrefcount(b), 2)
        del b
        # datum keeps a reference to data
        self.assertEqual(sys.getrefcount(data), 2)
        datum = data['Exif.Image.ImageDescription']
        self.assertEqual(sys.getrefcount(data), 3)
        # value keeps a reference to datum
        self.assertEqual(sys.getrefcount(datum), 2)
        v = datum.value()
        self.assertEqual(sys.getrefcount(datum), 3)
        del v
        self.assertEqual(sys.getrefcount(datum), 2)
        del datum
        self.assertEqual(sys.getrefcount(data), 2)
        del data
        self.assertEqual(sys.getrefcount(self.image), 2)


if __name__ == '__main__':
    unittest.main()



##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import io
import os
import sys
import unittest

import exiv2


class TestXmpModule(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        test_dir = os.path.dirname(__file__)
        # open image in memory so we don't corrupt the file
        with open(os.path.join(test_dir, 'image_02.jpg'), 'rb') as f:
            cls.image = exiv2.ImageFactory.open(f.read())
        # clear locale
        name = 'en_US.UTF-8'
        os.environ['LC_ALL'] = name
        os.environ['LANG'] = name
        os.environ['LANGUAGE'] = name

    def test_XmpData(self):
        # empty container
        data = exiv2.XmpData()
        self.assertEqual(len(data), 0)
        # actual data
        self.image.readMetadata()
        data = self.image.xmpData()
        self.assertEqual(len(data), 26)
        # add data
        data.add(exiv2.Xmpdatum(
            exiv2.XmpKey('Xmp.xmp.CreatorTool'), exiv2.AsciiValue('Acme')))
        self.assertEqual('Xmp.xmp.CreatorTool' in data, True)
        self.assertIsInstance(data['Xmp.xmp.CreatorTool'], exiv2.Xmpdatum)
        data.add(exiv2.XmpKey('Xmp.xmp.Nickname'), exiv2.AsciiValue('Pic'))
        self.assertEqual('Xmp.xmp.Nickname' in data, True)
        self.assertIsInstance(data['Xmp.xmp.Nickname'], exiv2.Xmpdatum)
        # iterators
        b = iter(data)
        self.assertIsInstance(b, exiv2.XmpData_iterator)
        self.assertEqual(b.key(), 'Xmp.iptc.CountryCode')
        b = data.begin()
        self.assertIsInstance(b, exiv2.XmpData_iterator)
        self.assertEqual(b.key(), 'Xmp.iptc.CountryCode')
        next(b)
        self.assertEqual(b.key(), 'Xmp.iptc.CreatorContactInfo')
        e = data.end()
        self.assertIsInstance(e, exiv2.XmpData_iterator_base)
        k = data.findKey(exiv2.XmpKey('Xmp.xmp.CreateDate'))
        self.assertIsInstance(k, exiv2.XmpData_iterator)
        self.assertEqual(k.key(), 'Xmp.xmp.CreateDate')
        k = data.erase(k)
        self.assertIsInstance(k, exiv2.XmpData_iterator)
        self.assertEqual(k.key(), 'Xmp.xmp.ModifyDate')
        self.assertEqual(len(data), 27)
        k = data.findKey(exiv2.XmpKey('Xmp.iptcExt.LocationCreated'))
        data.eraseFamily(k)
        self.assertEqual(len(data), 21)
        # access by key
        self.image.readMetadata()
        self.assertEqual('Xmp.dc.creator' in data, True)
        self.assertIsInstance(data['Xmp.dc.creator'], exiv2.Xmpdatum)
        del data['Xmp.dc.creator']
        self.assertEqual('Xmp.dc.creator' in data, False)
        data['Xmp.dc.creator'] = 'Fred'
        self.assertEqual('Xmp.dc.creator' in data, True)
        self.assertIsInstance(data['Xmp.dc.creator'], exiv2.Xmpdatum)
        with self.assertRaises(TypeError):
            data['Xmp.tiff.Orientation'] = 4
        data['Xmp.tiff.Orientation'] = exiv2.UShortValue(4)
        del data['Xmp.tiff.Orientation']
        b = data.begin()
        e = data.end()
        self.assertIsInstance(str(b), str)
        self.assertIsInstance(str(e), str)
        count = 0
        while b != e:
            next(b)
            count += 1
        self.assertEqual(count, 26)
        count = 0
        for d in data:
            count += 1
        self.assertEqual(count, 26)
        count = len(list(data))
        self.assertEqual(count, 26)
        # sorting
        data.sortByKey()
        self.assertEqual(data.begin().key(), 'Xmp.dc.creator')
        # other methods
        self.assertEqual(data.count(), 26)
        self.assertEqual(data.empty(), False)
        data.clear()
        self.assertEqual(len(data), 0)
        self.assertEqual(data.empty(), True)
        self.image.readMetadata()
        self.assertEqual(len(data), 26)
        self.assertIsInstance(data.xmpPacket(), str)
        data.setPacket('')
        self.assertEqual(data.usePacket(), False)
        self.assertEqual(data.usePacket(True), False)
        self.assertEqual(data.usePacket(), True)

    def _test_datum(self, datum):
        self.assertIsInstance(str(datum), str)
        buf = bytearray(datum.size())
        with self.assertRaises(exiv2.Exiv2Error) as cm:
            datum.copy(buf, exiv2.ByteOrder.littleEndian)
        self.assertEqual(cm.exception.code,
                         exiv2.ErrorCode.kerFunctionNotSupported)
        self.assertEqual(datum.count(), 3)
        self.assertEqual(datum.familyName(), 'Xmp')
        self.assertIsInstance(datum.getValue(), exiv2.LangAltValue)
        with self.assertWarns(DeprecationWarning):
            self.assertIsInstance(
                datum.getValue(exiv2.TypeId.langAlt), exiv2.LangAltValue)
        self.assertEqual(datum.groupName(), 'dc')
        self.assertEqual(datum.key(), 'Xmp.dc.description')
        self.assertEqual(datum.print(), 'lang="x-default" Good view of the'
            ' lighthouse., lang="en-GB" Good view of the lighthouse., lang="de"'
            ' Gute Sicht auf den Leuchtturm.')
        with self.assertWarns(DeprecationWarning):
            self.assertEqual(datum._print(), datum.print())
        self.assertEqual(datum.size(), 130)
        self.assertEqual(datum.tag(), 0)
        self.assertEqual(datum.tagLabel(), 'Description')
        self.assertEqual(datum.tagName(), 'description')
        self.assertEqual(datum.toFloat(0), 0.0)
        if exiv2.testVersion(0, 28, 0):
            self.assertEqual(datum.toInt64(0), 0)
        else:
            self.assertEqual(datum.toLong(0), 0)
        self.assertEqual(datum.toRational(0), (0, 0))
        self.assertEqual(
            datum.toString(), 'lang="x-default" Good view of the lighthouse.,'
            ' lang="en-GB" Good view of the lighthouse., lang="de" Gute Sicht'
            ' auf den Leuchtturm.')
        self.assertEqual(datum.toString(0), 'Good view of the lighthouse.')
        self.assertEqual(datum.typeId(), exiv2.TypeId.langAlt)
        self.assertEqual(datum.typeName(), 'LangAlt')
        self.assertEqual(datum.typeSize(), 0)
        self.assertIsInstance(datum.value(), exiv2.LangAltValue)
        with self.assertWarns(DeprecationWarning):
            self.assertIsInstance(
                datum.value(exiv2.TypeId.langAlt), exiv2.LangAltValue)
        buf = io.StringIO()
        buf = datum.write(buf)
        self.assertEqual(buf.getvalue(), datum.toString())
        datum.setValue('fred')
        datum.setValue(exiv2.XmpTextValue('Acme'))
        with self.assertRaises(TypeError):
            datum.setValue(123)

    def test_XmpData_iterator(self):
        self.image.readMetadata()
        data = self.image.xmpData()
        datum_iter = data.findKey(exiv2.XmpKey('Xmp.dc.description'))
        self.assertIsInstance(datum_iter, exiv2.XmpData_iterator)
        self.assertIsInstance(iter(datum_iter), exiv2.XmpData_iterator)
        self._test_datum(datum_iter)

    def test_XmpDatum(self):
        datum = exiv2.Xmpdatum(
            exiv2.XmpKey('Xmp.xmp.CreatorTool'), exiv2.XmpTextValue('Acme'))
        self.assertIsInstance(datum, exiv2.Xmpdatum)
        datum2 = exiv2.Xmpdatum(datum)
        self.assertIsInstance(datum2, exiv2.Xmpdatum)
        del datum2
        self.image.readMetadata()
        data = self.image.xmpData()
        datum = data['Xmp.dc.description']
        self.assertIsInstance(datum, exiv2.Xmpdatum)
        self._test_datum(datum)

    def test_ref_counts(self):
        self.image.readMetadata()
        # xmpData keeps a reference to image
        self.assertEqual(sys.getrefcount(self.image), 2)
        data = self.image.xmpData()
        self.assertEqual(sys.getrefcount(self.image), 3)
        # iterator keeps a reference to data
        self.assertEqual(sys.getrefcount(data), 2)
        b = data.begin()
        self.assertEqual(sys.getrefcount(data), 3)
        e = data.end()
        self.assertEqual(sys.getrefcount(data), 4)
        i = iter(data)
        self.assertEqual(sys.getrefcount(data), 5)
        k = data.findKey(exiv2.XmpKey('Xmp.xmp.Rating'))
        self.assertEqual(sys.getrefcount(data), 6)
        k2 = data.erase(k)
        self.assertEqual(sys.getrefcount(data), 7)
        del b, e, i, k, k2
        self.assertEqual(sys.getrefcount(data), 2)
        # iterator of an iterator keeps a reference to iterator
        b = data.begin()
        self.assertEqual(sys.getrefcount(b), 2)
        i = iter(b)
        self.assertEqual(sys.getrefcount(b), 3)
        del i
        self.assertEqual(sys.getrefcount(b), 2)
        # iterator value keeps a reference to iterator
        v = b.value()
        self.assertEqual(sys.getrefcount(b), 3)
        del v
        self.assertEqual(sys.getrefcount(b), 2)
        del b
        # datum keeps a reference to data
        self.assertEqual(sys.getrefcount(data), 2)
        datum = data['Xmp.dc.description']
        self.assertEqual(sys.getrefcount(data), 3)
        # value keeps a reference to datum
        self.assertEqual(sys.getrefcount(datum), 2)
        v = datum.value()
        self.assertEqual(sys.getrefcount(datum), 3)
        del v
        self.assertEqual(sys.getrefcount(datum), 2)
        del datum
        self.assertEqual(sys.getrefcount(data), 2)
        del data
        self.assertEqual(sys.getrefcount(self.image), 2)


if __name__ == '__main__':
    unittest.main()



# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.2.1
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
import exiv2.types

# Pull in all the attributes from the low-level C/C++ module
if __package__ or "." in __name__:
    from ._basicio import *
else:
    from _basicio import *


import enum

class PositionMeta(enum.EnumMeta):
    def __getattribute__(cls, name):
        obj = super().__getattribute__(name)
        if isinstance(obj, enum.Enum):
            import warnings
            warnings.warn(
                "Use 'BasicIo.Position' instead of 'Position'",
                DeprecationWarning)
        return obj

class DeprecatedPosition(enum.IntEnum, metaclass=PositionMeta):
    pass

Position = DeprecatedPosition('Position', _enum_list_Position())
Position.__doc__ = "Seek starting positions."





##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import io
import os
import sys
import unittest

import exiv2


class TestIptcModule(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        test_dir = os.path.dirname(__file__)
        # open image in memory so we don't corrupt the file
        with open(os.path.join(test_dir, 'image_02.jpg'), 'rb') as f:
            cls.image = exiv2.ImageFactory.open(f.read())

    def test_IptcData(self):
        # empty container
        data = exiv2.IptcData()
        self.assertEqual(len(data), 0)
        # actual data
        self.image.readMetadata()
        data = self.image.iptcData()
        self.assertEqual(len(data), 19)
        # add data
        data.add(exiv2.Iptcdatum(
            exiv2.IptcKey('Iptc.Application2.LocationCode'),
            exiv2.AsciiValue('XYZ')))
        self.assertEqual('Iptc.Application2.LocationCode' in data, True)
        self.assertIsInstance(
            data['Iptc.Application2.LocationCode'], exiv2.Iptcdatum)
        data.add(exiv2.IptcKey('Iptc.Application2.LocationName'),
                 exiv2.AsciiValue('Erewhon'))
        self.assertEqual('Iptc.Application2.LocationName' in data, True)
        self.assertIsInstance(
            data['Iptc.Application2.LocationName'], exiv2.Iptcdatum)
        # iterators
        b = iter(data)
        self.assertIsInstance(b, exiv2.IptcData_iterator)
        self.assertEqual(b.key(), 'Iptc.Envelope.CharacterSet')
        b = data.begin()
        self.assertIsInstance(b, exiv2.IptcData_iterator)
        self.assertEqual(b.key(), 'Iptc.Envelope.CharacterSet')
        next(b)
        self.assertEqual(b.key(), 'Iptc.Application2.Contact')
        e = data.end()
        self.assertIsInstance(e, exiv2.IptcData_iterator_base)
        k = data.findId(exiv2.IptcDataSets.SpecialInstructions)
        self.assertIsInstance(k, exiv2.IptcData_iterator)
        self.assertEqual(k.key(), 'Iptc.Application2.SpecialInstructions')
        k = data.findId(exiv2.IptcDataSets.SpecialInstructions,
                        exiv2.IptcDataSets.application2)
        self.assertIsInstance(k, exiv2.IptcData_iterator)
        self.assertEqual(k.key(), 'Iptc.Application2.SpecialInstructions')
        k = data.findKey(exiv2.IptcKey('Iptc.Application2.SpecialInstructions'))
        self.assertIsInstance(k, exiv2.IptcData_iterator)
        self.assertEqual(k.key(), 'Iptc.Application2.SpecialInstructions')
        k = data.erase(k)
        self.assertIsInstance(k, exiv2.IptcData_iterator)
        self.assertEqual(k.key(), 'Iptc.Application2.Keywords')
        b = data.begin()
        e = data.end()
        self.assertIsInstance(str(b), str)
        self.assertIsInstance(str(e), str)
        count = 0
        while b != e:
            next(b)
            count += 1
        self.assertEqual(count, 20)
        count = 0
        for d in data:
            count += 1
        self.assertEqual(count, 20)
        count = len(list(data))
        self.assertEqual(count, 20)
        # access by key
        self.assertEqual('Iptc.Application2.Byline' in data, True)
        self.assertIsInstance(data['Iptc.Application2.Byline'], exiv2.Iptcdatum)
        del data['Iptc.Application2.Byline']
        self.assertEqual('Iptc.Application2.Byline' in data, False)
        data['Iptc.Application2.Byline'] = 'Fred'
        self.assertEqual('Iptc.Application2.Byline' in data, True)
        self.assertIsInstance(data['Iptc.Application2.Byline'], exiv2.Iptcdatum)
        with self.assertRaises(TypeError):
            data['Iptc.Envelope.FileFormat'] = 2.5
        data['Iptc.Envelope.FileFormat'] = 4
        data['Iptc.Envelope.FileFormat'] = exiv2.UShortValue(4)
        del data['Iptc.Envelope.FileFormat']
        # sorting
        data.sortByKey()
        self.assertEqual(data.begin().key(), 'Iptc.Application2.Byline')
        data.sortByTag()
        self.assertEqual(data.begin().key(), 'Iptc.Application2.ObjectName')
        # other methods
        self.assertEqual(data.count(), 20)
        self.assertEqual(data.detectCharset(), 'UTF-8')
        self.assertEqual(data.empty(), False)
        self.assertEqual(data.size(), 400)
        data.clear()
        self.assertEqual(len(data), 0)
        self.assertEqual(data.empty(), True)

    def _test_datum(self, datum):
        self.assertIsInstance(str(datum), str)
        buf = bytearray(datum.count())
        self.assertEqual(
            datum.copy(buf, exiv2.ByteOrder.littleEndian), len(buf))
        self.assertEqual(buf, bytes(datum.toString(), 'ascii'))
        self.assertEqual(datum.count(), 28)
        self.assertEqual(datum.familyName(), 'Iptc')
        self.assertIsInstance(datum.getValue(), exiv2.StringValue)
        with self.assertWarns(DeprecationWarning):
            self.assertIsInstance(
                datum.getValue(exiv2.TypeId.string), exiv2.StringValue)
        self.assertEqual(datum.groupName(), 'Application2')
        self.assertEqual(datum.key(), 'Iptc.Application2.Caption')
        self.assertEqual(datum.print(), 'Good view of the lighthouse.')
        with self.assertWarns(DeprecationWarning):
            self.assertEqual(datum._print(), datum.print())
        self.assertEqual(datum.record(), exiv2.IptcDataSets.application2)
        self.assertEqual(datum.recordName(), 'Application2')
        self.assertEqual(datum.size(), 28)
        self.assertEqual(datum.tag(), exiv2.IptcDataSets.Caption)
        self.assertEqual(datum.tagLabel(), 'Caption')
        self.assertEqual(datum.tagName(), 'Caption')
        self.assertEqual(datum.toFloat(0), 71.0)
        if exiv2.testVersion(0, 28, 0):
            self.assertEqual(datum.toInt64(0), 71)
        else:
            self.assertEqual(datum.toLong(0), 71)
        self.assertEqual(datum.toRational(0), (71, 1))
        self.assertEqual(datum.toString(), 'Good view of the lighthouse.')
        self.assertEqual(datum.toString(0), 'Good view of the lighthouse.')
        self.assertEqual(datum.typeId(), exiv2.TypeId.string)
        self.assertEqual(datum.typeName(), 'String')
        self.assertEqual(datum.typeSize(), 1)
        self.assertIsInstance(datum.value(), exiv2.StringValue)
        with self.assertWarns(DeprecationWarning):
            self.assertIsInstance(
                datum.value(exiv2.TypeId.string), exiv2.StringValue)
        buf = io.StringIO()
        buf = datum.write(buf)
        self.assertEqual(buf.getvalue(), 'Good view of the lighthouse.')
        datum.setValue('fred')
        datum.setValue(exiv2.StringValue('Acme'))
        with self.assertRaises(TypeError):
            datum.setValue(123)

    def test_IptcData_iterator(self):
        self.image.readMetadata()
        data = self.image.iptcData()
        datum_iter = data.findKey(exiv2.IptcKey('Iptc.Application2.Caption'))
        self.assertIsInstance(datum_iter, exiv2.IptcData_iterator)
        self.assertIsInstance(iter(datum_iter), exiv2.IptcData_iterator)
        self._test_datum(datum_iter)

    def test_IptcDatum(self):
        datum = exiv2.Iptcdatum(
            exiv2.IptcKey('Iptc.Application2.Keywords'),
            exiv2.StringValue('holiday'))
        self.assertIsInstance(datum, exiv2.Iptcdatum)
        datum2 = exiv2.Iptcdatum(datum)
        self.assertIsInstance(datum2, exiv2.Iptcdatum)
        del datum2
        self.image.readMetadata()
        data = self.image.iptcData()
        datum = data['Iptc.Application2.Caption']
        self.assertIsInstance(datum, exiv2.Iptcdatum)
        self._test_datum(datum)

    def test_ref_counts(self):
        self.image.readMetadata()
        # iptcData keeps a reference to image
        self.assertEqual(sys.getrefcount(self.image), 2)
        data = self.image.iptcData()
        self.assertEqual(sys.getrefcount(self.image), 3)
        # iterator keeps a reference to data
        self.assertEqual(sys.getrefcount(data), 2)
        b = data.begin()
        self.assertEqual(sys.getrefcount(data), 3)
        e = data.end()
        self.assertEqual(sys.getrefcount(data), 4)
        i = iter(data)
        self.assertEqual(sys.getrefcount(data), 5)
        k = data.findKey(exiv2.IptcKey('Iptc.Application2.Caption'))
        self.assertEqual(sys.getrefcount(data), 6)
        k2 = data.erase(k)
        self.assertEqual(sys.getrefcount(data), 7)
        del b, e, i, k, k2
        self.assertEqual(sys.getrefcount(data), 2)
        # iterator of an iterator keeps a reference to iterator
        b = data.begin()
        self.assertEqual(sys.getrefcount(b), 2)
        i = iter(b)
        self.assertEqual(sys.getrefcount(b), 3)
        del i
        self.assertEqual(sys.getrefcount(b), 2)
        # iterator value keeps a reference to iterator
        v = b.value()
        self.assertEqual(sys.getrefcount(b), 3)
        del v
        self.assertEqual(sys.getrefcount(b), 2)
        del b
        # datum keeps a reference to data
        self.assertEqual(sys.getrefcount(data), 2)
        datum = data['Iptc.Application2.Contact']
        self.assertEqual(sys.getrefcount(data), 3)
        # value keeps a reference to datum
        self.assertEqual(sys.getrefcount(datum), 2)
        v = datum.value()
        self.assertEqual(sys.getrefcount(datum), 3)
        del v
        self.assertEqual(sys.getrefcount(datum), 2)
        del datum
        self.assertEqual(sys.getrefcount(data), 2)
        del data
        self.assertEqual(sys.getrefcount(self.image), 2)


if __name__ == '__main__':
    unittest.main()



##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import locale
import logging
import os
import random
import sys
import unittest

import exiv2


class TestTypesModule(unittest.TestCase):
    def check_result(self, result, expected_type, expected_value):
        self.assertIsInstance(result, expected_type)
        self.assertEqual(result, expected_value)

    def test_DataBuf(self):
        data = bytes(random.choices(range(256), k=128))
        # constructors
        buf = exiv2.DataBuf()
        self.assertIsInstance(buf, exiv2.DataBuf)
        self.assertEqual(len(buf), 0)
        buf = exiv2.DataBuf(4)
        self.assertEqual(len(buf), 4)
        buf = exiv2.DataBuf(data)
        self.assertEqual(len(buf), len(data))
        self.check_result(buf.data(), memoryview, data)
        # other methods
        self.check_result(buf.size(), int, len(data))
        with buf.data() as view:
            self.assertIsInstance(view, memoryview)
            self.check_result(view[23], int, data[23])
            view[49] = 99
            self.check_result(view[49], int, 99)
        buf = exiv2.DataBuf(data)
        self.assertEqual(buf, data)
        self.assertEqual(data, buf)
        self.assertNotEqual(buf, b'fred')
        self.assertNotEqual(b'fred', buf)
        if exiv2.testVersion(0, 28, 0):
            self.assertEqual(buf.cmpBytes(0, data), 0)
            self.assertEqual(buf.cmpBytes(5, data[5:]), 0)
            self.assertNotEqual(buf.cmpBytes(0, b'fred'), 0)
            buf.resize(6)
            self.assertEqual(len(buf), 6)
            self.assertEqual(buf.empty(), False)
            buf.resize(0)
            self.assertEqual(buf.empty(), True)
        else:
            with self.assertWarns(DeprecationWarning):
                result = buf[23]
            with self.assertWarns(DeprecationWarning):
                self.check_result(buf.pData_, memoryview, data)
            with self.assertWarns(DeprecationWarning):
                self.check_result(buf.size_, int, len(data))
            buf.free()
            self.assertEqual(len(buf), 0)
        buf = exiv2.DataBuf(data)
        buf.reset()
        self.assertEqual(len(buf), 0)
        buf = exiv2.DataBuf()
        buf.alloc(6)
        self.assertEqual(len(buf), 6)

    def test_Rational(self):
        for type_ in (exiv2.Rational, exiv2.URational):
            # constructors
            value = type_()
            self.assertIsInstance(value, type_)
            value = type_(13, 7)
            self.assertIsInstance(value, type_)
            # other methods
            self.assertEqual(len(value), 2)
            self.assertEqual(repr(value), '(13, 7)')
            self.check_result(value[0], int, 13)
            self.check_result(value[1], int, 7)
            self.check_result(value.first, int, value[0])
            self.check_result(value.second, int, value[1])
            value[0] = 23
            self.check_result(value[0], int, 23)

    def test_TypeInfo(self):
        info = exiv2.TypeInfo
        self.check_result(info.typeId('Rational'),
                          exiv2.TypeId, exiv2.TypeId.unsignedRational)
        self.check_result(info.typeName(exiv2.TypeId.unsignedRational),
                          str, 'Rational')
        self.check_result(info.typeSize(exiv2.TypeId.unsignedRational), int, 8)

    @unittest.skipUnless(exiv2.versionInfo()['EXV_ENABLE_NLS'],
                         'no localisation available')
    def test_localisation(self):
        str_en = 'Failed to read input data'
        str_de = 'Die Eingabedaten konnten nicht gelesen werden.'
        # clear current locale
        locale.setlocale(locale.LC_ALL, 'C')
        self.check_result(exiv2.exvGettext(str_en), str, str_en)
        # set German locale
        if sys.platform == 'win32':
            name = 'German'
        else:
            name = 'de_DE.UTF-8'
        try:
            locale.setlocale(locale.LC_ALL, name)
        except locale.Error:
            self.skipTest("failed to set locale")
            return
        name = 'de_DE.UTF-8'
        os.environ['LC_ALL'] = name
        os.environ['LANG'] = name
        os.environ['LANGUAGE'] = name
        locale.setlocale(locale.LC_ALL, '')
        name, encoding = locale.getlocale()
        if name != 'de_DE' and sys.platform != 'win32':
            self.skipTest("locale environment ignored")
        # test localisation
        self.check_result(exiv2.exvGettext(str_en), str, str_de)
        if exiv2.testVersion(0, 28, 3) or not exiv2.testVersion(0, 28, 0):
            with self.assertLogs(level=logging.WARNING) as cm:
                comment = exiv2.CommentValue('charset=invalid Fred')
            self.assertEqual(cm.output, [
                'WARNING:exiv2:Ungültiger Zeichensatz: "invalid"'])
            with self.assertRaises(exiv2.Exiv2Error) as cm:
                key = exiv2.ExifKey('not.a.tag')
            self.assertEqual(cm.exception.message,
                             "Ungültiger Schlüssel 'not.a.tag'")
        # clear locale
        name = 'en_US.UTF-8'
        os.environ['LC_ALL'] = name
        os.environ['LANG'] = name
        os.environ['LANGUAGE'] = name


if __name__ == '__main__':
    unittest.main()



# python-exiv2 - Python interface to exiv2
# http://github.com/jim-easterbrook/python-exiv2
# Copyright (C) 2024  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))


# -- Project information -----------------------------------------------------

project = 'python-exiv2'
copyright = '2004-2024 Exiv2 authors'
author = 'Jim Easterbrook'

import exiv2

# The full version, including alpha/beta/rc tags
release = '{} (exiv2 {})'.format(exiv2.__version__, exiv2.version())
version = exiv2.__version__

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = ['sphinx.ext.autodoc',
              'sphinx.ext.autosummary',
              'sphinx.ext.inheritance_diagram',
              'sphinx.ext.intersphinx']

autosummary_imported_members = True
autodoc_class_signature = 'separated'
autodoc_docstring_signature = False
autodoc_default_options = {
    'members': True,
    'undoc-members': True,
    'exclude-members': 'this, thisown, __init__, __new__',
    'show-inheritance': True,
    }

intersphinx_mapping = {
    'python': ('https://docs.python.org/3', None),
    }

add_module_names = False

import re

class_re = re.compile(r':class:`(\w+?)`')
percent_re = re.compile(r'%(\w+?)([.,|(\s]|$)')

def process_docstring(app, what, name, obj, options, lines):
    # replace/modify some docstrings
    if name == 'exiv2._version.__version__':
        lines[:] = ['python-exiv2 version as a string', '']
        return
    if name == 'exiv2._version.__version_tuple__':
        lines[:] = ['python-exiv2 version as a tuple of ints', '']
        return
    if 'iterator' in name and not lines:
        parts = name.split('.')
        lines[:] = ['See :meth:`{}.{}`.'.format(parts[2].replace(
            'Data_iterator', 'datum'), parts[3]), '']
        return
    # fixes for particular problems
    if name.endswith('error.LogMsg') or name.endswith('iptc.Iptcdatum'):
        # first line is not indented
        lines[0] = '       ' + lines[0]
    if name.endswith('XmpParser.initialize'):
        # fix broken code block comment parsing by SWIG
        in_code_block = False
        for idx in range(len(lines)):
            if not lines[idx]:
                continue
            if '.. code-block::' in lines[idx]:
                in_code_block = True
                indent = 8 + len(lines[idx]) - len(lines[idx].lstrip())
                indent = (' ' * indent)
                continue
            if ':rtype:' in lines[idx]:
                break
            if in_code_block and lines[idx][0] != ' ':
                lines[idx] = indent + '// ' + lines[idx]
    if name.endswith('ExifThumb.erase'):
        # fix %Thumbnail
        for idx in range(len(lines)):
            lines[idx] = lines[idx].replace(
                'Exif.%Thumbnail.*', '``Exif.Thumbnail.*``')
    if name.endswith('Io.read'):
        # fix size_ member
        for idx in range(len(lines)):
            lines[idx] = lines[idx].replace('DataBuf::size_', 'DataBuf.size\_')
    if name.endswith('XmpData.usePacket'):
        # fix usepacket_ member
        for idx in range(len(lines)):
            lines[idx] = lines[idx].replace('usePacket_', 'usePacket\_')
    # other substitutions
    for idx in range(len(lines)):
        line = lines[idx]
        line = line.replace(':raises: Error',
                            ':raises: :exc:`~exiv2.Exiv2Error` ')
        for match in class_re.finditer(line):
            part_name = match.group(1)
            try:
                obj = getattr(exiv2, part_name)
            except Exception as ex:
                break
            full_name = obj.__module__ + '.' + obj.__qualname__
            line = line.replace('`' + part_name + '`', '`~' + full_name + '`')
        for match in percent_re.finditer(line):
            part_name = match.group(1)
            line = line.replace('%' + part_name, '``' + part_name + '`` ')
        lines[idx] = line
    # remove arbitrary indentation
    remove = None
    for idx in range(len(lines)):
        line = lines[idx]
        active_line = line.lstrip()
        if not active_line:
            lines[idx] = ''
            continue
        indent = len(line) - len(active_line)
        if remove is None:
            remove = indent
        if active_line == '|' or active_line.startswith('*Overload'):
            # reset
            remove = None
        elif indent < remove:
            # unexpected unindentation
            print(name)
            print('unexpected unindentation', idx, line)
            remove = None
        else:
            lines[idx] = line[remove:]
    # convert to Sphinx-compatible reStructuredText
    idx = 0
    while idx < len(lines):
        line = lines[idx]
        if not line:
            idx += 1
            continue
        active_line = line.lstrip()
        indent = len(line) - len(active_line)
        min_indent = ' ' * (indent + 1)
        words = active_line.split()
        if words[0] == '..':
            # Sphinx directive such as '.. code::'
            # Copy until not indented
            while True:
                idx += 1
                if idx >= len(lines) or (
                        lines[idx] and not lines[idx].startswith(min_indent)):
                    break
            continue
        while min_indent and lines[idx+1].startswith(min_indent):
            # contenate indented continuation lines
            lines[idx] += ' ' + lines.pop(idx+1).lstrip()
        line = lines[idx]
        active_line = line.lstrip()
        if active_line == '|':
            # unneeded separator between overloads
            del lines[idx:idx+2]
            idx -= 2
        elif active_line[0] == '|':
            # probably a table
            parts = line.split('|')
            if len(parts) > 3 and parts[0].strip() == '' and parts[-1] == '':
                if idx > 0 and lines[idx-1]:
                    lines.insert(idx, '')
                    idx += 1
                header = ['"{}"'.format(x.strip('*')) for x in parts[1:-1]]
                lines[idx:idx+1] = [
                    parts[0] + '.. csv-table::', parts[0] + '    :delim: |',
                    parts[0] + '    :header: ' + ', '.join(header), '']
                idx += 3
                while lines[idx+1]:
                    idx += 1
                    parts = lines[idx].split('|')
                    lines[idx] = parts[0] + '    ' + '|'.join(parts[1:-1])
        elif words[0] in (':type', ':param', ':rtype:', ':return:', ':raises:',
                          ':ivar'):
            # insert a blank line above
            if idx > 0 and lines[idx-1]:
                lines.insert(idx, '')
                idx += 1
        elif words[0] == '``':
            # convert to a code block
            lines[idx:idx+1] = [lines[idx].replace('``', '.. code::'), '']
            idx += 1
            while lines[idx+1].strip() != '``':
                idx += 1
                lines[idx] = (' ' * 4) + lines[idx]
            del lines[idx+1]
        elif words[0] == '*Overload':
            # ensure next line is blank
            if lines[idx+1]:
                idx += 1
                lines.insert(idx, '')
        elif words[0] == 'WARNING:':
            lines[idx:idx+1] = ['', line.replace('WARNING:', '.. warning::')]
            idx += 1
        elif words[0] == 'Notes:':
            lines[idx:idx+1] = ['', line.replace('Notes:', '.. note::')]
            idx += 1
            while lines[idx+1]:
                idx += 1
                lines[idx] = min_indent + lines[idx].lstrip()
        idx += 1

def setup(app):
    app.connect('autodoc-process-docstring', process_docstring)

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']



##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import io
import os
import unittest

import exiv2


class TestTagsModule(unittest.TestCase):
    key_name = 'Exif.Image.ImageDescription'
    group_name = 'Image'
    tag = 270

    @classmethod
    def setUpClass(cls):
        # clear locale
        name = 'en_US.UTF-8'
        os.environ['LC_ALL'] = name
        os.environ['LANG'] = name
        os.environ['LANGUAGE'] = name

    def check_result(self, result, expected_type, expected_value):
        self.assertIsInstance(result, expected_type)
        self.assertEqual(result, expected_value)

    def test_ExifTags(self):
        tags = exiv2.ExifTags
        key = exiv2.ExifKey(self.key_name)
        self.check_result(tags.defaultCount(key), int, 0)
        group_list = tags.groupList()
        self.assertIsInstance(group_list, list)
        self.assertGreater(len(group_list), 0)
        self.assertIsInstance(group_list[0], exiv2.GroupInfo)
        self.check_result(tags.ifdName(self.group_name), str, 'IFD0')
        self.check_result(tags.isExifGroup(self.group_name), bool, True)
        self.check_result(tags.isMakerGroup(self.group_name), bool, False)
        self.check_result(tags.sectionName(key), str, 'OtherTags')
        tag_list = tags.tagList(self.group_name)
        self.assertIsInstance(tag_list, list)
        self.assertGreater(len(tag_list), 0)
        self.assertIsInstance(tag_list[0], exiv2.TagInfo)

    def test_GroupInfo(self):
        info = exiv2.ExifTags.groupList()[0]
        self.check_result(info['groupName'], str, 'Image')
        if exiv2.testVersion(0, 28, 0):
            self.check_result(info['ifdId'], int, exiv2.IfdId.ifd0Id)
        else:
            self.check_result(info['ifdId'], int, 1)
        self.check_result(info['ifdName'], str, 'IFD0')
        tag_list = info['tagList']()
        self.assertIsInstance(tag_list, list)
        self.assertGreater(len(tag_list), 0)
        self.assertIsInstance(tag_list[0], exiv2.TagInfo)

    def test_TagInfo(self):
        info = exiv2.ExifTags.tagList(self.group_name)[0]
        self.check_result(info['count'], int, 0)
        desc = info['desc']
        self.assertIsInstance(desc, str)
        self.assertTrue(desc.startswith('The name and version of the software'))
        if exiv2.testVersion(0, 28, 0):
            self.check_result(info['ifdId'], int, exiv2.IfdId.ifd0Id)
        else:
            self.check_result(info['ifdId'], int, 1)
        self.check_result(info['name'], str, 'ProcessingSoftware')
        if exiv2.testVersion(0, 28, 0):
            self.check_result(
                info['sectionId'], exiv2.SectionId, exiv2.SectionId.otherTags)
        else:
            self.check_result(info['sectionId'], int, 4)
        self.check_result(info['tag'], int, 11)
        self.check_result(info['title'], str, 'Processing Software')
        self.check_result(
            info['typeId'], exiv2.TypeId, exiv2.TypeId.asciiString)

    def test_ExifKey(self):
        # constructors
        key = exiv2.ExifKey(self.key_name)
        self.assertIsInstance(key, exiv2.ExifKey)
        key2 = exiv2.ExifKey(self.tag, self.group_name)
        self.assertIsInstance(key2, exiv2.ExifKey)
        self.assertIsNot(key2, key)
        key2 = exiv2.ExifKey(key)
        self.assertIsInstance(key2, exiv2.ExifKey)
        self.assertIsNot(key2, key)
        # other methods
        self.assertEqual(str(key), self.key_name)
        key2 = key.clone()
        self.assertIsNot(key2, key)
        self.assertIsInstance(key2, exiv2.ExifKey)
        self.check_result(
            key.defaultTypeId(), exiv2.TypeId, exiv2.TypeId.asciiString)
        self.check_result(key.familyName(), str, self.key_name.split('.')[0])
        self.check_result(key.groupName(), str, self.key_name.split('.')[1])
        self.check_result(key.idx(), int, 0)
        self.check_result(key.key(), str, self.key_name)
        key.setIdx(123)
        self.check_result(key.idx(), int, 123)
        self.check_result(key.tag(), int, self.tag)
        desc = key.tagDesc()
        self.assertIsInstance(desc, str)
        self.assertTrue(desc.startswith('A character string giving the title'))
        self.check_result(key.tagLabel(), str, 'Image Description')
        self.check_result(key.tagName(), str, self.key_name.split('.')[2])
        buf = io.StringIO()
        buf = key.write(buf)
        self.assertEqual(buf.getvalue(), self.key_name)


if __name__ == '__main__':
    unittest.main()



##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2022-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import os
import sys
import tempfile
import unittest

import exiv2


class TestPreviewModule(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        test_dir = os.path.dirname(__file__)
        cls.image_path = os.path.join(test_dir, 'image_02.jpg')
        # read image file data into memory
        with open(cls.image_path, 'rb') as f:
            cls.image_data = f.read()
        cls.image = exiv2.ImageFactory.open(cls.image_data)
        cls.image.readMetadata()

    def check_result(self, result, expected_type, expected_value):
        self.assertIsInstance(result, expected_type)
        self.assertEqual(result, expected_value)

    def test_PreviewImage(self):
        manager = exiv2.PreviewManager(self.image)
        props = manager.getPreviewProperties()
        preview = manager.getPreviewImage(props[0])
        self.assertIsInstance(preview, exiv2.PreviewImage)
        preview2 = exiv2.PreviewImage(preview)
        self.assertIsInstance(preview2, exiv2.PreviewImage)
        self.assertEqual(len(preview), preview.size())
        copy = preview.copy()
        self.assertIsInstance(copy, exiv2.DataBuf)
        with preview.pData() as data:
            self.check_result(data, memoryview, copy)
            self.assertEqual(data[:10], b'\xff\xd8\xff\xe0\x00\x10JFIF')
        self.assertEqual(memoryview(preview), copy)
        self.check_result(preview.extension(), str, '.jpg')
        self.check_result(preview.height(), int, 120)
        self.check_result(preview.id(), int, 4)
        self.check_result(preview.mimeType(), str, 'image/jpeg')
        self.check_result(preview.size(), int, 2532)
        self.check_result(preview.width(), int, 160)
        if not exiv2.versionInfo()['EXV_ENABLE_FILESYSTEM']:
            self.skipTest('EXV_ENABLE_FILESYSTEM is off')
        with tempfile.TemporaryDirectory() as tmp_dir:
            temp_file = os.path.join(tmp_dir, 'image.jpg')
            self.assertEqual(preview.writeFile(temp_file), 2532)

    def test_PreviewManager(self):
        manager = exiv2.PreviewManager(self.image)
        self.assertIsInstance(manager, exiv2.PreviewManager)
        props = manager.getPreviewProperties()
        self.assertIsInstance(props, tuple)
        self.assertEqual(len(props), 1)
        prop = props[0]
        self.assertIsInstance(prop, exiv2.PreviewProperties)
        preview = manager.getPreviewImage(prop)
        self.assertIsInstance(preview, exiv2.PreviewImage)

    def test_PreviewProperties(self):
        properties = exiv2.PreviewManager(self.image).getPreviewProperties()[0]
        self.assertIsInstance(properties, exiv2.PreviewProperties)
        self.check_result(properties.extension_, str, '.jpg')
        self.check_result(properties.height_, int, 120)
        self.check_result(properties.id_, int, 4)
        self.check_result(properties.mimeType_, str, 'image/jpeg')
        self.check_result(properties.size_, int, 2532)
        self.check_result(properties.width_, int, 160)
        keys = properties.keys()
        self.assertIsInstance(keys, list)
        self.assertEqual(len(keys), 6)
        values = properties.values()
        self.assertIsInstance(values, list)
        self.assertEqual(len(values), 6)
        items = properties.items()
        self.assertIsInstance(items, list)
        self.assertEqual(len(items), 6)
        for k in properties:
            v = properties[k]
            self.assertIn(k, keys)
            self.assertIn(v, values)
            self.assertIn((k, v), items)
            with self.assertRaises(TypeError):
                properties[k] = 123
            with self.assertRaises(TypeError):
                del properties[k]
        with self.assertRaises(KeyError):
            a = properties['fred']
        with self.assertRaises(KeyError):
            properties['fred'] = 123

    def test_ref_counts(self):
        # manager keeps reference to image
        self.assertEqual(sys.getrefcount(self.image), 2)
        manager = exiv2.PreviewManager(self.image)
        self.assertEqual(sys.getrefcount(self.image), 3)


if __name__ == '__main__':
    unittest.main()



##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import io
import os
import sys
import tempfile
import unittest

import exiv2


class TestDatasetsModule(unittest.TestCase):
    def check_result(self, result, expected_type, expected_value):
        self.assertIsInstance(result, expected_type)
        self.assertEqual(result, expected_value)

    def test_DataSet(self):
        dataset = exiv2.IptcDataSets.application2RecordList()[0]
        self.assertIsInstance(dataset, exiv2.DataSet)
        self.assertEqual(dataset['desc'][:27], 'A binary number identifying')
        self.check_result(dataset['mandatory'], bool, True)
        self.check_result(dataset['maxbytes'], int, 2)
        self.check_result(dataset['minbytes'], int, 2)
        self.check_result(dataset['name'], str, 'RecordVersion')
        self.check_result(dataset['number'], int, 0)
        self.check_result(dataset['photoshop'], str, '')
        self.check_result(dataset['recordId'],
                          int, exiv2.IptcDataSets.application2)
        self.check_result(dataset['repeatable'], bool, False)
        self.check_result(dataset['title'], str, 'Record Version')
        self.check_result(dataset['type'],
                          exiv2.TypeId, exiv2.TypeId.unsignedShort)

    def test_IptcDataSets(self):
        # number and record id of Iptc.Application2.Caption
        number = exiv2.IptcDataSets.Caption
        record_id = exiv2.IptcDataSets.application2
        # static data lists
        datasets = exiv2.IptcDataSets.application2RecordList()
        self.assertIsInstance(datasets, list)
        self.assertIsInstance(datasets[0], exiv2.DataSet)
        datasets = exiv2.IptcDataSets.envelopeRecordList()
        self.assertIsInstance(datasets, list)
        self.assertIsInstance(datasets[0], exiv2.DataSet)
        # test other methods 
        self.check_result(exiv2.IptcDataSets.dataSet('Caption', record_id),
                          int, exiv2.IptcDataSets.Caption)
        self.check_result(exiv2.IptcDataSets.dataSetDesc(number, record_id),
                          str, 'A textual description of the object data.')
        self.check_result(exiv2.IptcDataSets.dataSetName(number, record_id),
                          str, 'Caption')
        self.check_result(exiv2.IptcDataSets.dataSetPsName(number, record_id),
                          str, 'Description')
        self.check_result(exiv2.IptcDataSets.dataSetRepeatable(
            number, record_id), bool, False)
        self.check_result(exiv2.IptcDataSets.dataSetTitle(
            number, record_id), str, 'Caption')
        self.check_result(exiv2.IptcDataSets.dataSetType(
            number, record_id), exiv2.TypeId, exiv2.TypeId.string)
        self.check_result(exiv2.IptcDataSets.recordDesc(
            record_id), str, 'IIM application record 2')
        self.check_result(exiv2.IptcDataSets.recordId('Application2'),
                          int, exiv2.IptcDataSets.application2)
        self.check_result(exiv2.IptcDataSets.recordName(
            exiv2.IptcDataSets.application2), str, 'Application2')

    def test_IptcKey(self):
        key_name = 'Iptc.Application2.Caption'
        # constructors
        key = exiv2.IptcKey(key_name)
        self.assertIsInstance(key, exiv2.IptcKey)
        key2 = exiv2.IptcKey(key.tag(), key.record())
        self.assertIsInstance(key2, exiv2.IptcKey)
        self.check_result(key2.key(), str, key_name)
        # copy
        key2 = key.clone()
        self.check_result(key2.key(), str, key_name)
        # other methods
        self.check_result(key.familyName(), str, key_name.split('.')[0])
        self.check_result(key.groupName(), str, key_name.split('.')[1])
        self.check_result(key.key(), str, key_name)
        self.check_result(key.record(), int, exiv2.IptcDataSets.application2)
        self.check_result(key.recordName(), str, key_name.split('.')[1])
        self.check_result(key.tag(), int, exiv2.IptcDataSets.Caption)
        self.check_result(key.tagLabel(), str, key_name.split('.')[2])
        self.check_result(key.tagName(), str, key_name.split('.')[2])
        buf = io.StringIO()
        buf = key.write(buf)
        self.assertEqual(buf.getvalue(), key_name)


if __name__ == '__main__':
    unittest.main()



# python-exiv2 - Python interface to exiv2
# http://github.com/jim-easterbrook/python-exiv2
# Copyright (C) 2024  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys

from sphinx.application import Sphinx


def main(argv=None):
    root = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
    src_dir = os.path.join(root, 'src', 'doc')
    dst_dir = os.path.join(root, 'doc', 'html')
    doctree_dir = os.path.join(root, 'doc', 'doctree')
    app = Sphinx(src_dir, src_dir, dst_dir, doctree_dir, 'html')
    app.build()
    return 0


if __name__ == "__main__":
    sys.exit(main())



# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.2.1
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
import exiv2.types

# Pull in all the attributes from the low-level C/C++ module
if __package__ or "." in __name__:
    from ._value import *
else:
    from _value import *


import enum

class CharsetIdMeta(enum.EnumMeta):
    def __getattribute__(cls, name):
        obj = super().__getattribute__(name)
        if isinstance(obj, enum.Enum):
            import warnings
            warnings.warn(
                "Use 'CommentValue.CharsetId' instead of 'CharsetId'",
                DeprecationWarning)
        return obj

class DeprecatedCharsetId(enum.IntEnum, metaclass=CharsetIdMeta):
    pass

CharsetId = DeprecatedCharsetId('CharsetId', _enum_list_CharsetId())
CharsetId.__doc__ = "Character set identifiers for the character sets defined by Exif."


import enum

class XmpArrayTypeMeta(enum.EnumMeta):
    def __getattribute__(cls, name):
        obj = super().__getattribute__(name)
        if isinstance(obj, enum.Enum):
            import warnings
            warnings.warn(
                "Use 'XmpValue.XmpArrayType' instead of 'XmpArrayType'",
                DeprecationWarning)
        return obj

class DeprecatedXmpArrayType(enum.IntEnum, metaclass=XmpArrayTypeMeta):
    pass

XmpArrayType = DeprecatedXmpArrayType('XmpArrayType', _enum_list_XmpArrayType())
XmpArrayType.__doc__ = "XMP array types."


import enum

class XmpStructMeta(enum.EnumMeta):
    def __getattribute__(cls, name):
        obj = super().__getattribute__(name)
        if isinstance(obj, enum.Enum):
            import warnings
            warnings.warn(
                "Use 'XmpValue.XmpStruct' instead of 'XmpStruct'",
                DeprecationWarning)
        return obj

class DeprecatedXmpStruct(enum.IntEnum, metaclass=XmpStructMeta):
    pass

XmpStruct = DeprecatedXmpStruct('XmpStruct', _enum_list_XmpStruct())
XmpStruct.__doc__ = "XMP structure indicator."





##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import enum
import logging
import os
import unittest

import exiv2


class TestErrorModule(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        # clear locale
        name = 'en_US.UTF-8'
        os.environ['LC_ALL'] = name
        os.environ['LANG'] = name
        os.environ['LANGUAGE'] = name

    def check_result(self, result, expected_type, expected_value):
        self.assertIsInstance(result, expected_type)
        self.assertEqual(result, expected_value)

    def test_LogMsg(self):
        self.assertIsInstance(exiv2.LogMsg.Level, enum.EnumMeta)
        self.check_result(
            exiv2.LogMsg.level(), exiv2.LogMsg.Level, exiv2.LogMsg.Level.warn)
        exiv2.LogMsg.setLevel(exiv2.LogMsg.Level.debug)
        self.check_result(
            exiv2.LogMsg.level(), exiv2.LogMsg.Level, exiv2.LogMsg.Level.debug)
        # get exiv2 to log a message
        with self.assertLogs(level=logging.WARNING):
            comment = exiv2.CommentValue('charset=invalid Fred')
        # test setting and clearing handler
        self.assertEqual(exiv2.LogMsg.handler(), exiv2.LogMsg.pythonHandler)
        exiv2.LogMsg.setHandler(None)
        self.assertEqual(exiv2.LogMsg.handler(), None)
        exiv2.LogMsg.setHandler(exiv2.LogMsg.defaultHandler)
        self.assertEqual(exiv2.LogMsg.handler(), exiv2.LogMsg.defaultHandler)
        exiv2.LogMsg.setHandler(exiv2.LogMsg.pythonHandler)
        self.assertEqual(exiv2.LogMsg.handler(), exiv2.LogMsg.pythonHandler)
        # get exiv2 to raise an exception
        with self.assertRaises(exiv2.Exiv2Error) as cm:
            image = exiv2.ImageFactory.open(bytes())
        self.assertEqual(cm.exception.code,
                         exiv2.ErrorCode.kerInputDataReadFailed)


if __name__ == '__main__':
    unittest.main()



# python-exiv2 - Python interface to libexiv2
# http://github.com/jim-easterbrook/python-exiv2
# Copyright (C) 2021-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup, Extension
from setuptools import __version__ as setuptools_version
import os
import re
import subprocess
import sys


def pkg_config(library, option):
    cmd = ['pkg-config', '--' + option, library]
    try:
        return subprocess.check_output(cmd, universal_newlines=True).strip()
    except Exception as ex:
        print(str(ex))
        return None

def get_version(inc_dir):
    with open(os.path.join(inc_dir, 'exiv2', 'exv_conf.h')) as cnf:
        for line in cnf.readlines():
            words = line.split()
            if len(words) < 3:
                continue
            if words[0] == '#define' and words[1] == 'EXV_PACKAGE_VERSION':
                return [int(x) for x in eval(words[2]).split('.')]
    return [0, 0]

# get list of available swigged versions
swigged_versions = []
for name in os.listdir('src'):
    parts = name.split('-')
    if parts[0] == 'swig' and parts[1] not in swigged_versions:
        swigged_versions.append([int(x) for x in parts[1].split('_')])
    swigged_versions.sort()

def get_mod_src_dir(exiv2_version):
    if len(exiv2_version) < 3:
        exiv2_version += [0]
    swigged_versions.sort()
    for v in swigged_versions:
        if v >= exiv2_version and v[:2] == exiv2_version[:2]:
            return os.path.join('src', 'swig-{}_{}_{}'.format(*v))
    swigged_versions.sort(reverse=True)
    for v in swigged_versions:
        if v[:2] == exiv2_version[:2]:
            return os.path.join('src', 'swig-{}_{}_{}'.format(*v))
    return None

mod_src_dir = None
platform = sys.platform
if platform == 'win32' and 'GCC' in sys.version:
    platform = 'mingw'

packages = ['exiv2', 'exiv2.examples']
package_dir = {'exiv2.examples': 'examples'}
package_data = {'exiv2.examples': ['*.py', '*.rst']}

if 'EXIV2_ROOT' in os.environ:
    # use local copy of libexiv2
    exiv2_root = os.path.normpath(os.environ['EXIV2_ROOT'])
    # header files
    path = os.path.join(exiv2_root, 'include')
    if not os.path.isfile(os.path.join(path, 'exiv2', 'exiv2.hpp')):
        print('ERROR: Include files not found')
        sys.exit(1)
    include_dirs = [path]
    # library files
    packages.append('exiv2.lib')
    if platform == 'linux':
        path = os.path.join(exiv2_root, 'lib64')
        if not os.path.exists(path):
            path = os.path.join(exiv2_root, 'lib')
        library_dirs = [path]
        package_dir['exiv2.lib'] = path
        package_data['exiv2.lib'] = [x for x in os.listdir(path)
                                     if re.fullmatch('libexiv2\.so\.\d+', x)]
    elif platform == 'darwin':
        path = os.path.join(exiv2_root, 'lib')
        library_dirs = [path]
        package_dir['exiv2.lib'] = path
        package_data['exiv2.lib'] = [x for x in os.listdir(path)
                                     if re.fullmatch('libexiv2\.\d+\.dylib', x)]
    elif platform in ('win32', 'mingw'):
        library_dirs = [os.path.join(exiv2_root, 'lib')]
        package_dir['exiv2.lib'] = os.path.join(exiv2_root, 'bin')
        package_data['exiv2.lib'] = ['*.dll']
    if not os.path.isdir(package_dir['exiv2.lib']):
        print('ERROR: Library files not found')
        sys.exit(1)
    # locale files
    path = os.path.join(exiv2_root, 'share', 'locale')
    if not os.path.isdir(path):
        print('WARNING: Locale files not found')
    else:
        packages.append('exiv2.locale')
        package_dir['exiv2.locale'] = path
        package_data['exiv2.locale'] = ['*/LC_MESSAGES/exiv2.mo']
        for name in os.listdir(path):
            if os.path.isfile(os.path.join(
                    path, name, 'LC_MESSAGES', 'exiv2.mo')):
                packages.append('exiv2.locale.' + name + '.' + 'LC_MESSAGES')
    # get exiv2 version from include files
    exiv2_version = get_version(include_dirs[0])
    mod_src_dir = get_mod_src_dir(exiv2_version)
    if platform == 'linux':
        extra_link_args = ['-Wl,-rpath,$ORIGIN/lib']
    elif platform == 'darwin':
        extra_link_args = ['-Wl,-rpath,@loader_path/lib']
    else:
        extra_link_args = []
else:
    # use installed libexiv2
    exiv2_version = pkg_config('exiv2', 'modversion')
    if exiv2_version:
        exiv2_version = [int(x) for x in exiv2_version.split('.')]
        mod_src_dir = get_mod_src_dir(exiv2_version)
        if mod_src_dir:
            library_dirs = pkg_config('exiv2', 'libs-only-L').split('-L')
            library_dirs = [x.strip() for x in library_dirs]
            library_dirs = [x.replace(r'\ ', ' ') for x in library_dirs if x]
            include_dirs = pkg_config('exiv2', 'cflags-only-I').split('-I')
            include_dirs = [x.strip() for x in include_dirs]
            include_dirs = [x.replace(r'\ ', ' ') for x in include_dirs if x]
            extra_link_args = []

if not mod_src_dir:
    print('ERROR: No SWIG source for libexiv2 version {}'.format(exiv2_version))
    sys.exit(1)

print('Using libexiv2 v{} with SWIG files from {}'.format(
    '.'.join(map(str, exiv2_version)), mod_src_dir))

package_dir['exiv2'] = mod_src_dir

# create extension modules list
ext_modules = []
extra_compile_args = []
define_macros = [('PY_SSIZE_T_CLEAN', None),
                 ('SWIG_TYPE_TABLE', 'exiv2')]
if platform in ('linux', 'darwin', 'mingw'):
    extra_compile_args = [
        '-O3', '-Wno-unused-variable', '-Wno-unused-function',
        '-Wno-deprecated-declarations', '-Wno-deprecated']
    if platform in ['linux', 'mingw']:
        extra_compile_args.append('-Wno-unused-but-set-variable')
    if 'PYTHON_EXIV2_STRICT' in os.environ:
        extra_compile_args.append('-Werror')
    if exiv2_version >= [0, 28]:
        extra_compile_args.append('-std=gnu++17')
    else:
        extra_compile_args.append('-std=c++98')
if platform == 'win32':
    extra_compile_args = ['/wd4101', '/wd4290']
if platform == 'darwin':
    cmd = ['brew', '--prefix']
    try:
        prefix = subprocess.check_output(cmd, universal_newlines=True).strip()
        include_dirs.append(os.path.join(prefix, 'include'))
        library_dirs.append(os.path.join(prefix, 'lib'))
    except Exception as ex:
        print(str(ex))
for file_name in os.listdir(mod_src_dir):
    if file_name[-9:] != '_wrap.cxx':
        continue
    ext_name = file_name[:-9]
    ext_modules.append(Extension(
        '_' + ext_name,
        sources = [os.path.join(mod_src_dir, file_name)],
        include_dirs = include_dirs,
        extra_compile_args = extra_compile_args,
        define_macros = define_macros,
        libraries = ['exiv2'],
        library_dirs = library_dirs,
        extra_link_args = extra_link_args,
        ))

setup_kwds = {
    'ext_package': 'exiv2',
    'ext_modules': ext_modules,
    'packages': packages,
    'package_dir': package_dir,
    'package_data': package_data,
    'exclude_package_data': {'exiv2': ['*.cxx']},
    }

if tuple(map(int, setuptools_version.split('.')[:2])) < (61, 0):
    # get metadata from pyproject.toml
    import toml
    metadata = toml.load('pyproject.toml')

    with open(metadata['project']['readme']) as ldf:
        long_description = ldf.read()
    py_exiv2_version = long_description.splitlines()[0].split()[-1]

    setup_kwds.update(
        name = metadata['project']['name'],
        version = py_exiv2_version,
        description = metadata['project']['description'],
        long_description = long_description,
        author = metadata['project']['authors'][0]['name'],
        author_email = metadata['project']['authors'][0]['email'],
        url = metadata['project']['urls']['Homepage'],
        classifiers = metadata['project']['classifiers'],
        platforms = metadata['tool']['setuptools']['platforms'],
        license = metadata['project']['license']['text'],
        zip_safe = metadata['tool']['setuptools']['zip-safe'],
        )

setup(**setup_kwds)



##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2023  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import unittest

import exiv2


class TestVersionModule(unittest.TestCase):
    def test_module(self):
        # earliest usable libexiv2 version
        result = exiv2.testVersion(0, 27, 0)
        self.assertIsInstance(result, bool)
        self.assertEqual(result, True)
        version = exiv2.version()
        self.assertIsInstance(version, str)
        self.assertGreaterEqual(version, '0.27.0')
        version_tuple = tuple(int(x) for x in version.split('.')[:3])
        version = exiv2.versionNumber()
        self.assertIsInstance(version, int)
        self.assertEqual(version, (version_tuple[0] << 16)
                         + (version_tuple[1] << 8) + version_tuple[2])
        version = exiv2.versionNumberHexString()
        self.assertIsInstance(version, str)
        self.assertEqual(
            version, '{:02x}{:02x}{:02x}'.format(*version_tuple))
        version = exiv2.versionString()
        self.assertIsInstance(version, str)
        self.assertGreaterEqual(
            version, '.'.join(str(x) for x in version_tuple))


if __name__ == '__main__':
    unittest.main()



# python-exiv2 - Python interface to exiv2
# http://github.com/jim-easterbrook/python-exiv2
# Copyright (C) 2021-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import re
import shutil
import subprocess
import sys
import tempfile


def get_version(incl_dir):
    with open(os.path.join(incl_dir, 'exv_conf.h')) as cnf:
        for line in cnf.readlines():
            words = line.split()
            if len(words) < 3:
                continue
            if words[0] == '#define' and words[1] == 'EXV_PACKAGE_VERSION':
                version = [int(x) for x in eval(words[2]).split('.')]
                return tuple(version + [0, 0])
    return 0, 0, 0, 0


def main():
    # get SWIG version
    cmd = ['swig', '-version']
    try:
        swig_version = str(subprocess.Popen(
            cmd, stdout=subprocess.PIPE,
            universal_newlines=True).communicate()[0])
    except Exception:
        print('ERROR: command "%s" failed' % ' '.join(cmd))
        raise
    for line in swig_version.splitlines():
        if 'Version' in line:
            swig_version = tuple(map(int, line.split()[-1].split('.')))
            break
    if swig_version < (4, 1, 0):
        print('SWIG version 4.1.0 or later required')
        return 1
    # get source to SWIG
    if len(sys.argv) != 2:
        print('Usage: %s path' % sys.argv[0])
        return 1
    # get config
    platform = sys.platform
    if platform == 'win32' and 'GCC' in sys.version:
        platform = 'mingw'
    incl_dir = os.path.normpath(sys.argv[1])
    if os.path.basename(incl_dir) != 'exiv2':
        incl_dir = os.path.join(incl_dir, 'exiv2')
    if not os.path.isdir(incl_dir):
        print('Directory %s not found' % incl_dir)
        return 2
    if 'exiv2.hpp' not in os.listdir(incl_dir):
        print('Exiv2 header files not found in %s' % incl_dir)
        return 3
    # get exiv2 version
    exiv2_version = get_version(incl_dir)
    # get exiv2 build options
    options = {}
    with open(os.path.join(incl_dir, 'exv_conf.h')) as cnf:
        for line in cnf.readlines():
            words = line.split()
            if len(words) < 2:
                continue
            if words[0] == '#define' and words[1].startswith('EXV_'):
                options[words[1]] = ' '.join(words[2:]) or None
    # get python-exiv2 version
    with open('README.rst') as rst:
        py_exiv2_version = rst.readline().split()[-1]
    # get list of modules (Python) and extensions (SWIG)
    interface_dir = os.path.join('src', 'interface')
    file_names = os.listdir(interface_dir)
    file_names.sort()
    file_names = [os.path.splitext(x) for x in file_names]
    mod_names = [x[0] + x[1] for x in file_names if x[1] == '.py']
    ext_names = [x[0] for x in file_names if x[1] == '.i']
    # convert exiv2 version to hex
    exiv2_version_hex = '0x{:02x}{:02x}{:02x}{:02x}'.format(*exiv2_version)
    # create output dir
    output_dir = os.path.join('src', 'swig-{}_{}_{}'.format(*exiv2_version))
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
    # copy Python modules
    for mod_name in mod_names:
        shutil.copy2(os.path.join('src', 'interface', mod_name),
                     os.path.join(output_dir, mod_name))
    # pre-process include files to a temporary directory
    subst = {
        'basicio.hpp': [('/*isWriteable*/', 'isWriteable')],
        'image.hpp': [('getType(const byte* data, size_t size',
                       'getType(const byte* data, size_t A'),
                      ('getType(const byte* data, long size',
                       'getType(const byte* data, long A'),
                      ('open(const byte* data, size_t size',
                       'open(const byte* data, size_t B'),
                      ('open(const byte* data, long size',
                       'open(const byte* data, long B'),
                      ('/*! @brief', '/*!\n    @brief')],
        'metadatum.hpp': [('toString(size_t n)', 'toString(size_t i)'),
                          ('toString(long n)', 'toString(long i)')],
        'preview.hpp': [('_{};', '_;')],
        }
    for key in ('exif.hpp', 'iptc.hpp', 'value.hpp',
                'xmp.hpp', 'xmp_exiv2.hpp'):
        subst[key] = subst['metadatum.hpp']
    if swig_version < (4, 2, 0):
        subst['basicio.hpp'].append(('static constexpr auto',
                                     'static const char*'))
    with tempfile.TemporaryDirectory() as copy_dir:
        dest = os.path.join(copy_dir, 'exiv2')
        os.makedirs(dest)
        for file in os.listdir(incl_dir):
            if file in subst:
                with open(os.path.join(incl_dir, file), 'r') as in_file:
                    with open(os.path.join(dest, file), 'w') as out_file:
                        for line in in_file.readlines():
                            for from_to in subst[file]:
                                line = line.replace(*from_to)
                            out_file.write(line)
            else:
                shutil.copy(os.path.join(incl_dir, file),
                            os.path.join(dest, file))
        # make options list
        swig_opts = ['-c++', '-python', '-builtin', '-doxygen',
                     '-fastdispatch', '-fastproxy', '-Wextra', '-Werror',
                     '-DEXIV2_VERSION_HEX=' + exiv2_version_hex]
        for k, v in options.items():
            if v is None:
                swig_opts.append('-D{}'.format(k))
            else:
                swig_opts.append('-D{}={}'.format(k, v))
        swig_opts += ['-I' + copy_dir, '-outdir', output_dir]
        # do each swig module
        for ext_name in ext_names:
            cmd = ['swig'] + swig_opts
            # Functions with just one parameter and a default value don't
            # work with fastunpack.
            # See https://github.com/swig/swig/issues/2786
            if swig_version < (4, 2, 2) and ext_name in (
                    'basicio', 'exif', 'iptc', 'metadatum', 'value', 'xmp'):
                cmd.append('-nofastunpack')
            cmd += ['-o', os.path.join(output_dir, ext_name + '_wrap.cxx')]
            cmd += [os.path.join(interface_dir, ext_name + '.i')]
            print(' '.join(cmd))
            subprocess.check_call(cmd)
    # create init module
    init_file = os.path.join(output_dir, '__init__.py')
    with open(init_file, 'w') as im:
        im.write(f'''
import os
import sys

if sys.platform == 'win32':
    _dir = os.path.join(os.path.dirname(__file__), 'lib')
    if os.path.isdir(_dir):
        if hasattr(os, 'add_dll_directory'):
            os.add_dll_directory(_dir)
        os.environ['PATH'] = _dir + ';' + os.environ['PATH']

class Exiv2Error(Exception):
    """Python exception raised by exiv2 library errors.

    :ivar ErrorCode code: The Exiv2 error code that caused the exception.
    :ivar str message: The message associated with the exception.
    """
    def __init__(self, code, message):
        self.code= code
        self.message = message

#: python-exiv2 version as a string
__version__ = "{py_exiv2_version}"
#: python-exiv2 version as a tuple of ints
__version_tuple__ = tuple(({', '.join(re.split(r'[-.]', py_exiv2_version))}))

__all__ = ["Exiv2Error"]
''')
        for name in ext_names:
            im.write(f'from exiv2.{name} import *\n')
            im.write(f'__all__ += exiv2._{name}.__all__\n')
        im.write("""
__all__ = [x for x in __all__ if x[0] != '_']
__all__.sort()
""")
    # update ReadTheDocs config
    with open('src/doc/requirements.txt', 'w') as f:
        f.write(f'''exiv2 <= {py_exiv2_version}
sphinx == 7.2.6
sphinx-rtd-theme == 2.0.0
''')
    return 0


if __name__ == "__main__":
    sys.exit(main())



##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2023-24  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import os
import unittest

import exiv2


class TestEasyaccessModule(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        test_dir = os.path.dirname(__file__)
        with open(os.path.join(test_dir, 'image_02.jpg'), 'rb') as f:
            image = exiv2.ImageFactory.open(f.read())
        image.readMetadata()
        cls.exif_data = image.exifData()

    def check_result(self, datum, expected_type=None):
        # not all files have a value
        if datum is not None:
            self.assertIsInstance(datum, exiv2.Exifdatum)
            self.assertIsInstance(datum.getValue(), expected_type)

    def test_easyaccess(self):
        self.check_result(exiv2.afPoint(self.exif_data))
        self.check_result(exiv2.contrast(self.exif_data))
        self.check_result(exiv2.exposureMode(self.exif_data))
        self.check_result(exiv2.exposureTime(self.exif_data))
        self.check_result(exiv2.fNumber(self.exif_data), exiv2.URationalValue)
        self.check_result(exiv2.flashBias(self.exif_data))
        self.check_result(exiv2.focalLength(self.exif_data), exiv2.URationalValue)
        self.check_result(exiv2.imageQuality(self.exif_data))
        self.check_result(exiv2.isoSpeed(self.exif_data))
        self.check_result(exiv2.lensName(self.exif_data), exiv2.AsciiValue)
        self.check_result(exiv2.macroMode(self.exif_data))
        self.check_result(exiv2.make(self.exif_data))
        self.check_result(exiv2.meteringMode(self.exif_data))
        self.check_result(exiv2.model(self.exif_data))
        self.check_result(exiv2.orientation(self.exif_data), exiv2.UShortValue)
        self.check_result(exiv2.saturation(self.exif_data))
        self.check_result(exiv2.sceneCaptureType(self.exif_data))
        self.check_result(exiv2.sceneMode(self.exif_data))
        self.check_result(exiv2.serialNumber(self.exif_data))
        self.check_result(exiv2.sharpness(self.exif_data))
        self.check_result(exiv2.subjectDistance(self.exif_data))
        self.check_result(exiv2.whiteBalance(self.exif_data))
        if not exiv2.testVersion(0, 27, 4):
            self.skipTest('easyaccess funcs introduced in v0.27.4')
        self.check_result(exiv2.apertureValue(self.exif_data), exiv2.URationalValue)
        self.check_result(exiv2.brightnessValue(self.exif_data))
        self.check_result(exiv2.dateTimeOriginal(self.exif_data), exiv2.AsciiValue)
        self.check_result(exiv2.exposureBiasValue(self.exif_data))
        self.check_result(exiv2.exposureIndex(self.exif_data))
        self.check_result(exiv2.flash(self.exif_data))
        self.check_result(exiv2.flashEnergy(self.exif_data))
        self.check_result(exiv2.lightSource(self.exif_data))
        self.check_result(exiv2.maxApertureValue(self.exif_data))
        self.check_result(exiv2.sensingMethod(self.exif_data))
        self.check_result(exiv2.shutterSpeedValue(self.exif_data))
        self.check_result(exiv2.subjectArea(self.exif_data))


if __name__ == '__main__':
    unittest.main()
